@@ -1116,14 +1116,14 @@ HomeLens::~HomeLens()
   delete pimpl;
 }
 
-void HomeLens::AddLenses(Lenses& lenses)
+void HomeLens::AddLenses(Lenses::Ptr const& lenses)
 {
-  for (auto lens : lenses.GetLenses())
+  for (auto lens : lenses->GetLenses())
   {
     pimpl->OnLensAdded(lens);
   }
 
-  lenses.lens_added.connect(sigc::mem_fun(pimpl, &HomeLens::Impl::OnLensAdded));
+  lenses->lens_added.connect(sigc::mem_fun(pimpl, &HomeLens::Impl::OnLensAdded));
 }
 
 Lenses::LensList HomeLens::GetLenses() const
@@ -1158,6 +1158,11 @@ Lens::Ptr HomeLens::GetLensAtIndex(std::size_t index) const
   return Lens::Ptr();
 }
 
+Lens::Ptr HomeLens::GetLensForShortcut(std::string const& lens_shortcut) const
+{
+  return Lens::Ptr();
+}
+
 void HomeLens::GlobalSearch(std::string const& search_string, SearchFinishedCallback const& cb)
 {
   LOG_WARN(logger) << "Global search not enabled for HomeLens class."
