@@ -208,7 +208,8 @@ void ResultViewGrid::Activate(std::string const& uri, int index, ResultView::Act
   }
 
   active_index_ = index;
-  glib::Variant data(g_variant_new("(iiiiii)", column_x, row_y, column_width, row_height, left_results, right_results));
+  guint64 timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+  glib::Variant data(g_variant_new("(tiiiiii)", timestamp, column_x, row_y, column_width, row_height, left_results, right_results));
   UriActivated.emit(uri, type, data);
 }
 
@@ -757,7 +758,6 @@ void ResultViewGrid::DrawContent(nux::GraphicsEngine& GfxContext, bool force_dra
 
   if (GetCompositionLayout())
   {
-    nux::Geometry geo = GetCompositionLayout()->GetGeometry();
     GetCompositionLayout()->ProcessDraw(GfxContext, force_draw);
   }
 
