@@ -32,6 +32,7 @@
 #include "FavoriteStore.h"
 #include "Launcher.h"
 #include "MultiMonitor.h"
+#include "unity-shared/GnomeFileManager.h"
 #include "unity-shared/UBusMessages.h"
 
 #include <glib/gi18n-lib.h>
@@ -51,14 +52,17 @@ const std::string WINDOW_MOVE_TIMEOUT = "bamf-window-move";
 const std::string ICON_REMOVE_TIMEOUT = "bamf-icon-remove";
 //const std::string ICON_DND_OVER_TIMEOUT = "bamf-icon-dnd-over";
 const std::string DEFAULT_ICON = "application-default-icon";
+const int MAXIMUM_QUICKLIST_WIDTH = 300;
 }
 
 NUX_IMPLEMENT_OBJECT_TYPE(ApplicationLauncherIcon);
 
 ApplicationLauncherIcon::ApplicationLauncherIcon(ApplicationPtr const& app)
   : SimpleLauncherIcon(IconType::APPLICATION)
-  , app_(app)
   , _startup_notification_timestamp(0)
+  , _last_scroll_timestamp(0)
+  , _last_scroll_direction(ScrollDirection::DOWN)
+  , _progressive_scroll(0)
   , use_custom_bg_color_(false)
   , bg_color_(nux::color::White)
 {
@@ -83,71 +87,7 @@ ApplicationLauncherIcon::ApplicationLauncherIcon(ApplicationPtr const& app)
     << ", active: " << (app->active() ? "yes" : "no")
     << ", running: " << (app->running() ? "yes" : "no");
 
-  // Lambda functions should be fine here because when the application the icon
-  // is only ever removed when the application is closed.
-  app->window_opened.connect([this](ApplicationWindow const&) {
-                               EnsureWindowState();
-                               UpdateMenus();
-                               UpdateIconGeometries(GetCenters());
-                             });
-  app->window_closed.connect([this]() { EnsureWindowState(); });
-  app->window_moved.connect([this](ApplicationWindow const&) { EnsureWindowState(); });
-
-  app->urgent.changed.connect([this](bool const& urgent) {
-                            LOG_DEBUG(logger) << tooltip_text() << " urgent now " << (urgent ? "true" : "false");
-                            SetQuirk(Quirk::URGENT, urgent);
-                          });
-  app->active.changed.connect([this](bool const& active) {
-                            LOG_DEBUG(logger) << tooltip_text() << " active now " << (active ? "true" : "false");
-                            SetQuirk(Quirk::ACTIVE, active);
-                          });
-  app->running.changed.connect([this](bool const& running) {
-                            LOG_DEBUG(logger) << tooltip_text() << " running now " << (running ? "true" : "false");
-                            SetQuirk(Quirk::RUNNING, running);
-
-                            if (running)
-                            {
-                              _source_manager.Remove(ICON_REMOVE_TIMEOUT);
-
-                              /* It can happen that these values are not set
-                               * during initialization if the view is closed
-                               * very early, so we need to make sure that they
-                               * are updated as soon as the view is re-opened. */
-                              if (tooltip_text().empty())
-                                tooltip_text = app_->title();
-
-                              if (icon_name == DEFAULT_ICON)
-                              {
-                                std::string icon = app_->icon();
-                                icon_name = (icon.empty() ? DEFAULT_ICON : icon);
-                              }
-
-                              EnsureWindowState();
-                              UpdateIconGeometries(GetCenters());
-                            }
-                          });
-  app->visible.changed.connect([this](bool const& visible) {
-                            if (!IsSticky())
-                              SetQuirk(Quirk::VISIBLE, visible);
-                          });
-
-  app->closed.connect([this]() {
-                            if (!IsSticky())
-                            {
-                              SetQuirk(Quirk::VISIBLE, false);
-
-                              /* Use a timeout to remove the icon, this avoids
-                               * that we remove an application that is going
-                               * to be reopened soon. So applications that
-                               * have a splash screen won't be removed from
-                               * the launcher while the splash is closed and
-                               * a new window is opened. */
-                              _source_manager.AddTimeoutSeconds(1, [&] {
-                                Remove();
-                                return false;
-                              }, ICON_REMOVE_TIMEOUT);
-                            }
-                          });
+  SetApplication(app);
 
   WindowManager& wm = WindowManager::Default();
   wm.window_minimized.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::OnWindowMinimized));
@@ -171,6 +111,103 @@ ApplicationLauncherIcon::~ApplicationLauncherIcon()
     app_->sticky = false;
     app_->seen = false;
   }
+
+  DisconnectApplicationSignalsConnections();
+}
+
+void ApplicationLauncherIcon::SetApplication(ApplicationPtr const& app)
+{
+  if (app_ == app)
+    return;
+
+  app_ = app;
+  DisconnectApplicationSignalsConnections();
+  SetupApplicationSignalsConnections();
+}
+
+void ApplicationLauncherIcon::SetupApplicationSignalsConnections()
+{
+  // Lambda functions should be fine here because when the application the icon
+  // is only ever removed when the application is closed.
+  window_opened_connection_ = app_->window_opened.connect([this](ApplicationWindow const&) {
+    EnsureWindowState();
+    UpdateMenus();
+    UpdateIconGeometries(GetCenters());
+  });
+
+  window_closed_connection_ = app_->window_closed.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState));
+  window_moved_connection_ = app_->window_moved.connect(sigc::hide(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState)));
+
+  urgent_changed_connection_ = app_->urgent.changed.connect([this](bool const& urgent) {
+    LOG_DEBUG(logger) << tooltip_text() << " urgent now " << (urgent ? "true" : "false");
+    SetQuirk(Quirk::URGENT, urgent);
+  });
+
+  active_changed_connection_ = app_->active.changed.connect([this](bool const& active) {
+    LOG_DEBUG(logger) << tooltip_text() << " active now " << (active ? "true" : "false");
+    SetQuirk(Quirk::ACTIVE, active);
+  });
+
+  running_changed_connection_ = app_->running.changed.connect([this](bool const& running) {
+    LOG_DEBUG(logger) << tooltip_text() << " running now " << (running ? "true" : "false");
+    SetQuirk(Quirk::RUNNING, running);
+
+    if (running)
+    {
+      _source_manager.Remove(ICON_REMOVE_TIMEOUT);
+
+      /* It can happen that these values are not set
+       * during initialization if the view is closed
+       * very early, so we need to make sure that they
+       * are updated as soon as the view is re-opened. */
+      if (tooltip_text().empty())
+        tooltip_text = app_->title();
+
+      if (icon_name == DEFAULT_ICON)
+      {
+        std::string icon = app_->icon();
+        icon_name = (icon.empty() ? DEFAULT_ICON : icon);
+      }
+
+      EnsureWindowState();
+      UpdateIconGeometries(GetCenters());
+    }
+  });
+
+  visible_changed_connection_ = app_->visible.changed.connect([this](bool const& visible) {
+    if (!IsSticky())
+      SetQuirk(Quirk::VISIBLE, visible);
+  });
+
+  closed_changed_connection_ = app_->closed.connect([this]() {
+    if (!IsSticky())
+    {
+      SetQuirk(Quirk::VISIBLE, false);
+
+      /* Use a timeout to remove the icon, this avoids
+       * that we remove an application that is going
+       * to be reopened soon. So applications that
+       * have a splash screen won't be removed from
+       * the launcher while the splash is closed and
+       * a new window is opened. */
+      _source_manager.AddTimeoutSeconds(1, [this] {
+        Remove();
+        return false;
+      }, ICON_REMOVE_TIMEOUT);
+    }
+  });
+}
+
+void ApplicationLauncherIcon::DisconnectApplicationSignalsConnections()
+{
+  window_opened_connection_.disconnect();
+  window_closed_connection_.disconnect();
+  window_moved_connection_.disconnect();
+  urgent_changed_connection_.disconnect();
+  active_changed_connection_.disconnect();
+  running_changed_connection_.disconnect();
+  visible_changed_connection_.disconnect();
+  closed_changed_connection_.disconnect();
 }
 
 bool ApplicationLauncherIcon::GetQuirk(AbstractLauncherIcon::Quirk quirk) const
@@ -258,7 +295,7 @@ void ApplicationLauncherIcon::ActivateLauncherIcon(ActionArg arg)
    * an unmapped (!= minimized) window around and
    * if so force "Focus" behaviour */
 
-  if (arg.source != ActionArg::SWITCHER)
+  if (arg.source != ActionArg::Source::SWITCHER)
   {
     user_visible = app_->visible();
 
@@ -307,6 +344,28 @@ void ApplicationLauncherIcon::ActivateLauncherIcon(ActionArg arg)
       if (any_on_monitor && arg.monitor >= 0 && active_monitor != arg.monitor)
         active = false;
     }
+
+    if (user_visible && IsSticky() && IsFileManager())
+    {
+      // See bug #753938
+      unsigned minimum_windows = 0;
+      auto const& file_manager = GnomeFileManager::Get();
+
+      if (file_manager->IsTrashOpened())
+        ++minimum_windows;
+
+      if (file_manager->IsDeviceOpened())
+        ++minimum_windows;
+
+      if (minimum_windows > 0)
+      {
+        if (file_manager->OpenedLocations().size() == minimum_windows &&
+            GetWindows(WindowFilter::USER_VISIBLE|WindowFilter::MAPPED).size() == minimum_windows)
+        {
+          user_visible = false;
+        }
+      }
+    }
   }
 
   /* Behaviour:
@@ -323,7 +382,7 @@ void ApplicationLauncherIcon::ActivateLauncherIcon(ActionArg arg)
       return;
 
     SetQuirk(Quirk::STARTING, true);
-    OpenInstanceLauncherIcon();
+    OpenInstanceLauncherIcon(arg.timestamp);
   }
   else // app is running
   {
@@ -335,7 +394,7 @@ void ApplicationLauncherIcon::ActivateLauncherIcon(ActionArg arg)
       }
       else // #2 above
       {
-        if (arg.source != ActionArg::SWITCHER)
+        if (arg.source != ActionArg::Source::SWITCHER)
         {
           Spread(true, 0, false);
         }
@@ -346,7 +405,7 @@ void ApplicationLauncherIcon::ActivateLauncherIcon(ActionArg arg)
       if (scaleWasActive) // #4 above
       {
         Focus(arg);
-        if (arg.source != ActionArg::SWITCHER)
+        if (arg.source != ActionArg::Source::SWITCHER)
           Spread(true, 0, false);
       }
       else // #3 above
@@ -439,7 +498,7 @@ void ApplicationLauncherIcon::OnWindowMoved(guint32 moved_win)
   if (!app_->OwnsWindow(moved_win))
     return;
 
-  _source_manager.AddTimeout(250, [&] {
+  _source_manager.AddTimeout(250, [this] {
     EnsureWindowState();
     UpdateIconGeometries(GetCenters());
 
@@ -449,7 +508,7 @@ void ApplicationLauncherIcon::OnWindowMoved(guint32 moved_win)
 
 void ApplicationLauncherIcon::UpdateDesktopFile()
 {
-  std::string filename = app_->desktop_file();
+  std::string const& filename = app_->desktop_file();
 
   if (!filename.empty() && _desktop_file != filename)
   {
@@ -464,24 +523,32 @@ void ApplicationLauncherIcon::UpdateDesktopFile()
     glib::Object<GFile> desktop_file(g_file_new_for_path(_desktop_file.c_str()));
     _desktop_file_monitor = g_file_monitor_file(desktop_file, G_FILE_MONITOR_NONE,
                                                 nullptr, nullptr);
-    g_file_monitor_set_rate_limit(_desktop_file_monitor, 1000);
+    g_file_monitor_set_rate_limit(_desktop_file_monitor, 2000);
 
-    auto sig = new glib::Signal<void, GFileMonitor*, GFile*, GFile*, GFileMonitorEvent>(_desktop_file_monitor, "changed",
-                                [&] (GFileMonitor*, GFile*, GFile*, GFileMonitorEvent event_type) {
-                                  switch (event_type)
-                                  {
-                                    case G_FILE_MONITOR_EVENT_DELETED:
-                                      UnStick();
-                                      break;
-                                    case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
-                                      UpdateDesktopQuickList();
-                                      UpdateBackgroundColor();
-                                      break;
-                                    default:
-                                      break;
-                                  }
-                                });
-    _gsignals.Add(sig);
+    _gsignals.Add<void, GFileMonitor*, GFile*, GFile*, GFileMonitorEvent>(_desktop_file_monitor, "changed",
+      [this] (GFileMonitor*, GFile* f, GFile*, GFileMonitorEvent event_type) {
+      switch (event_type)
+      {
+        case G_FILE_MONITOR_EVENT_DELETED:
+        {
+          glib::Object<GFile> file(f, glib::AddRef());
+          _source_manager.AddTimeoutSeconds(1, [this, file] {
+            if (!g_file_query_exists (file, nullptr))
+              UnStick();
+            return false;
+          });
+          break;
+        }
+        case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+        {
+          UpdateDesktopQuickList();
+          UpdateBackgroundColor();
+          break;
+        }
+        default:
+          break;
+      }
+    });
   }
 }
 
@@ -509,7 +576,7 @@ void ApplicationLauncherIcon::AddProperties(GVariantBuilder* builder)
     .add("startup_notification_timestamp", _startup_notification_timestamp);
 }
 
-void ApplicationLauncherIcon::OpenInstanceWithUris(std::set<std::string> const& uris)
+void ApplicationLauncherIcon::OpenInstanceWithUris(std::set<std::string> const& uris, Time timestamp)
 {
   glib::Error error;
   glib::Object<GDesktopAppInfo> desktopInfo(g_desktop_app_info_new_from_filename(DesktopFile().c_str()));
@@ -518,8 +585,9 @@ void ApplicationLauncherIcon::OpenInstanceWithUris(std::set<std::string> const&
   GdkDisplay* display = gdk_display_get_default();
   glib::Object<GdkAppLaunchContext> app_launch_context(gdk_display_get_app_launch_context(display));
 
-  _startup_notification_timestamp = nux::GetWindowThread()->GetGraphicsDisplay().GetCurrentEvent().x11_timestamp;
-  gdk_app_launch_context_set_timestamp(app_launch_context, _startup_notification_timestamp);
+  _startup_notification_timestamp = timestamp;
+  if (_startup_notification_timestamp > 0)
+    gdk_app_launch_context_set_timestamp(app_launch_context, _startup_notification_timestamp);
 
   if (g_app_info_supports_uris(appInfo))
   {
@@ -557,10 +625,10 @@ void ApplicationLauncherIcon::OpenInstanceWithUris(std::set<std::string> const&
   UpdateQuirkTime(Quirk::STARTING);
 }
 
-void ApplicationLauncherIcon::OpenInstanceLauncherIcon()
+void ApplicationLauncherIcon::OpenInstanceLauncherIcon(Time timestamp)
 {
   std::set<std::string> empty;
-  OpenInstanceWithUris(empty);
+  OpenInstanceWithUris(empty, timestamp);
 }
 
 void ApplicationLauncherIcon::Focus(ActionArg arg)
@@ -575,11 +643,11 @@ void ApplicationLauncherIcon::Focus(ActionArg arg)
   else if (app_->type() == "webapp")
   {
     // Webapps are again special.
-    OpenInstanceLauncherIcon();
+    OpenInstanceLauncherIcon(arg.timestamp);
     return;
   }
 
-  bool show_only_visible = arg.source == ActionArg::SWITCHER;
+  bool show_only_visible = arg.source == ActionArg::Source::SWITCHER;
   app_->Focus(show_only_visible, arg.monitor);
 }
 
@@ -633,7 +701,7 @@ void ApplicationLauncherIcon::UpdateDesktopQuickList()
   {
     for (GList *l = dbusmenu_menuitem_get_children(_menu_desktop_shortcuts); l; l = l->next)
     {
-      _gsignals.Disconnect(l->data, "item-activated");
+      _gsignals.Disconnect(l->data, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED);
     }
   }
 
@@ -660,15 +728,16 @@ void ApplicationLauncherIcon::UpdateDesktopQuickList()
     dbusmenu_menuitem_property_set(item, DBUSMENU_MENUITEM_PROP_LABEL, name);
     dbusmenu_menuitem_property_set_bool(item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
     dbusmenu_menuitem_property_set_bool(item, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
-    dbusmenu_menuitem_property_set(item, "shortcut-nick", nicks[index]);
+    std::string nick(nicks[index]);
 
-    auto sig = new glib::Signal<void, DbusmenuMenuitem*, gint>(item, "item-activated",
-                                [&] (DbusmenuMenuitem* item, gint) {
-                                  const gchar *nick;
-                                  nick = dbusmenu_menuitem_property_get(item, "shortcut-nick");
-                                  indicator_desktop_shortcuts_nick_exec(_desktop_shortcuts, nick);
-                                });
-    _gsignals.Add(sig);
+    _gsignals.Add<void, DbusmenuMenuitem*, gint>(item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
+    [this, nick] (DbusmenuMenuitem* item, unsigned timestamp) {
+      GdkDisplay* display = gdk_display_get_default();
+      glib::Object<GdkAppLaunchContext> context(gdk_display_get_app_launch_context(display));
+      gdk_app_launch_context_set_timestamp(context, timestamp);
+      auto gcontext = glib::object_cast<GAppLaunchContext>(context);
+      indicator_desktop_shortcuts_nick_exec_with_context(_desktop_shortcuts, nick.c_str(), gcontext);
+    });
 
     dbusmenu_menuitem_child_append(_menu_desktop_shortcuts, item);
     index++;
@@ -692,6 +761,50 @@ void ApplicationLauncherIcon::UpdateBackgroundColor()
     EmitNeedsRedraw();
 }
 
+void ApplicationLauncherIcon::EnsureMenuItemsWindowsReady()
+{
+  // delete all menu items for windows
+  _menu_items_windows.clear();
+
+  auto const& windows = Windows();
+
+  // We only add quicklist menu-items for windows if we have more than one window
+  if (windows.size() < 2)
+    return;
+   
+  Window active = WindowManager::Default().GetActiveWindow();
+
+  // add menu items for all open windows
+  for (auto const& w : windows)
+  {
+    if (w->title().empty())
+      continue;
+
+    glib::Object<DbusmenuMenuitem> menu_item(dbusmenu_menuitem_new());
+    dbusmenu_menuitem_property_set(menu_item, DBUSMENU_MENUITEM_PROP_LABEL, w->title().c_str());
+    dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_ENABLED, true);
+    dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_VISIBLE, true);
+    dbusmenu_menuitem_property_set_bool(menu_item, QuicklistMenuItem::MARKUP_ACCEL_DISABLED_PROPERTY, true);
+    dbusmenu_menuitem_property_set_int(menu_item, QuicklistMenuItem::MAXIMUM_LABEL_WIDTH_PROPERTY, MAXIMUM_QUICKLIST_WIDTH);
+
+    Window xid = w->window_id();
+    _gsignals.Add<void, DbusmenuMenuitem*, unsigned>(menu_item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
+      [xid] (DbusmenuMenuitem*, unsigned) {
+        WindowManager& wm = WindowManager::Default();
+        wm.Activate(xid);
+        wm.Raise(xid);
+    });
+    
+    if (xid == active)
+    {
+      dbusmenu_menuitem_property_set(menu_item, DBUSMENU_MENUITEM_PROP_TOGGLE_TYPE, DBUSMENU_MENUITEM_TOGGLE_RADIO);
+      dbusmenu_menuitem_property_set_int(menu_item, DBUSMENU_MENUITEM_PROP_TOGGLE_STATE, DBUSMENU_MENUITEM_TOGGLE_STATE_CHECKED);
+    }
+
+    _menu_items_windows.push_back(menu_item);
+  }
+}
+
 void ApplicationLauncherIcon::UpdateMenus()
 {
   // add dynamic quicklist
@@ -742,7 +855,10 @@ void ApplicationLauncherIcon::UnStick()
   if (!IsSticky())
     return;
 
+  SetQuirk(Quirk::VISIBLE, app_->running());
+
   app_->sticky = false;
+
   if (!app_->running())
     Remove();
 }
@@ -770,8 +886,8 @@ void ApplicationLauncherIcon::EnsureMenuItemsReady()
     dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_ENABLED, true);
     dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_VISIBLE, true);
 
-    _gsignals.Add<void, DbusmenuMenuitem*, int>(menu_item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
-      [&] (DbusmenuMenuitem*, int) {
+    _gsignals.Add<void, DbusmenuMenuitem*, unsigned>(menu_item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
+      [this] (DbusmenuMenuitem*, unsigned) {
         ToggleSticky();
     });
 
@@ -791,8 +907,8 @@ void ApplicationLauncherIcon::EnsureMenuItemsReady()
     dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_ENABLED, true);
     dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_VISIBLE, true);
 
-    _gsignals.Add<void, DbusmenuMenuitem*, int>(menu_item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
-      [&] (DbusmenuMenuitem*, int) {
+    _gsignals.Add<void, DbusmenuMenuitem*, unsigned>(menu_item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
+      [this] (DbusmenuMenuitem*, unsigned) {
         Quit();
     });
 
@@ -885,24 +1001,19 @@ AbstractLauncherIcon::MenuItemsVector ApplicationLauncherIcon::GetMenus()
   else
   {
     glib::String app_name(g_markup_escape_text(app_->title().c_str(), -1));
-    std::ostringstream bold_app_name;
-    bold_app_name << "<b>" << app_name << "</b>";
+    std::string bold_app_name("<b>"+app_name.Str()+"</b>");
 
     item = dbusmenu_menuitem_new();
     dbusmenu_menuitem_property_set(item,
                                    DBUSMENU_MENUITEM_PROP_LABEL,
-                                   bold_app_name.str().c_str());
-    dbusmenu_menuitem_property_set_bool(item,
-                                        DBUSMENU_MENUITEM_PROP_ENABLED,
-                                        true);
-    dbusmenu_menuitem_property_set_bool(item,
-                                        QuicklistMenuItem::MARKUP_ENABLED_PROPERTY,
-                                        true);
+                                   bold_app_name.c_str());
+    dbusmenu_menuitem_property_set_bool(item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
+    dbusmenu_menuitem_property_set_bool(item, QuicklistMenuItem::MARKUP_ENABLED_PROPERTY, TRUE);
 
-    _gsignals.Add<void, DbusmenuMenuitem*, int>(item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
-      [&] (DbusmenuMenuitem*, int) {
-        _source_manager.AddIdle([&] {
-          ActivateLauncherIcon(ActionArg());
+    _gsignals.Add<void, DbusmenuMenuitem*, unsigned>(item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
+      [this] (DbusmenuMenuitem*, unsigned timestamp) {
+        _source_manager.AddIdle([this, timestamp] {
+          ActivateLauncherIcon(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
           return false;
         });
     });
@@ -926,6 +1037,28 @@ AbstractLauncherIcon::MenuItemsVector ApplicationLauncherIcon::GetMenus()
   }
   result.push_back(item);
 
+  EnsureMenuItemsWindowsReady();
+
+  if (!_menu_items_windows.empty())
+  {
+    for (auto const& it : _menu_items_windows)
+      result.push_back(it);
+
+    auto third_sep = _menu_items_extra.find("ThirdSeparator");
+    if (third_sep != _menu_items_extra.end())
+    {
+      item = third_sep->second;
+    }
+    else
+    {
+      item = dbusmenu_menuitem_new();
+      dbusmenu_menuitem_property_set(item, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
+      _menu_items_extra["ThirdSeparator"] = glib::Object<DbusmenuMenuitem>(item);
+    }
+
+    result.push_back(item);
+  }
+
   EnsureMenuItemsReady();
 
   for (auto it : _menu_items)
@@ -1034,7 +1167,7 @@ void ApplicationLauncherIcon::OnDndHovered()
 void ApplicationLauncherIcon::OnDndEnter()
 {
   /* Disabled, since the DND code is currently disabled as well.
-  _source_manager.AddTimeout(1000, [&] {
+  _source_manager.AddTimeout(1000, [this] {
     OnDndHovered();
     return false;
   }, ICON_DND_OVER_TIMEOUT);
@@ -1048,12 +1181,18 @@ void ApplicationLauncherIcon::OnDndLeave()
   */
 }
 
+bool ApplicationLauncherIcon::IsFileManager()
+{
+  auto const& desktop_file = DesktopFile();
+
+  return boost::algorithm::ends_with(desktop_file, "nautilus.desktop") ||
+         boost::algorithm::ends_with(desktop_file, "nautilus-folder-handler.desktop") ||
+         boost::algorithm::ends_with(desktop_file, "nautilus-home.desktop");
+}
+
 bool ApplicationLauncherIcon::OnShouldHighlightOnDrag(DndData const& dnd_data)
 {
-  bool is_home_launcher = boost::algorithm::ends_with(DesktopFile(), "nautilus-home.desktop") ||
-                          boost::algorithm::ends_with(DesktopFile(), "nautilus.desktop");
-
-  if (is_home_launcher)
+  if (IsFileManager())
   {
     return true;
   }
@@ -1084,7 +1223,8 @@ nux::DndAction ApplicationLauncherIcon::OnQueryAcceptDrop(DndData const& dnd_dat
 
 void ApplicationLauncherIcon::OnAcceptDrop(DndData const& dnd_data)
 {
-  OpenInstanceWithUris(ValidateUrisForLaunch(dnd_data));
+  auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+  OpenInstanceWithUris(ValidateUrisForLaunch(dnd_data), timestamp);
 }
 
 bool ApplicationLauncherIcon::ShowInSwitcher(bool current)
@@ -1100,7 +1240,7 @@ bool ApplicationLauncherIcon::ShowInSwitcher(bool current)
     }
     else
     {
-      for (int i = 0; i < max_num_monitors; i++)
+      for (int i = 0; i < max_num_monitors; ++i)
       {
         if (WindowVisibleOnMonitor(i))
         {
@@ -1114,6 +1254,11 @@ bool ApplicationLauncherIcon::ShowInSwitcher(bool current)
   return result;
 }
 
+bool ApplicationLauncherIcon::AllowDetailViewInSwitcher() const
+{
+  return app_->type() != "webapp";
+}
+
 unsigned long long ApplicationLauncherIcon::SwitcherPriority()
 {
   unsigned long long result = 0;
@@ -1151,6 +1296,94 @@ const std::set<std::string> ApplicationLauncherIcon::GetSupportedTypes()
   return supported_types;
 }
 
+void PerformScrollUp(WindowList const& windows, unsigned int progressive_scroll)
+{
+  if (!progressive_scroll)
+  {
+    windows.at(1)->Focus();
+    return;
+  }
+  else if (progressive_scroll == 1)
+  {
+    windows.back()->Focus();
+    return;
+  }
+
+  WindowManager::Default().RestackBelow(windows.at(0)->window_id(), windows.at(windows.size() - progressive_scroll + 1)->window_id());
+  windows.at(windows.size() - progressive_scroll + 1)->Focus();
+}
+
+void PerformScrollDown(WindowList const& windows, unsigned int progressive_scroll)
+{
+  if (!progressive_scroll)
+  {
+    WindowManager::Default().RestackBelow(windows.at(0)->window_id(), windows.back()->window_id());
+    windows.at(1)->Focus();
+    return;
+  }
+
+  WindowManager::Default().RestackBelow(windows.at(0)->window_id(), windows.at(progressive_scroll)->window_id());
+  windows.at(progressive_scroll)->Focus();
+}
+
+void ApplicationLauncherIcon::PerformScroll(ScrollDirection direction, Time timestamp)
+{
+  if (timestamp - _last_scroll_timestamp < 150)
+    return;
+  else if (timestamp - _last_scroll_timestamp > 1500 || direction != _last_scroll_direction)
+    _progressive_scroll = 0;
+
+  _last_scroll_timestamp = timestamp;
+  _last_scroll_direction = direction;
+
+  auto const& windows = GetWindowsOnCurrentDesktopInStackingOrder();
+  if (windows.empty())
+    return;
+
+  if (!IsActive())
+  {
+    windows.at(0)->Focus();
+    return;
+  }
+
+  if (windows.size() <= 1)
+    return; 
+
+  ++_progressive_scroll;
+  _progressive_scroll %= windows.size();
+
+  switch(direction)
+  {
+  case ScrollDirection::UP:
+    PerformScrollUp(windows, _progressive_scroll);
+    break;
+  case ScrollDirection::DOWN:
+    PerformScrollDown(windows, _progressive_scroll);
+    break;
+  }
+}
+
+WindowList ApplicationLauncherIcon::GetWindowsOnCurrentDesktopInStackingOrder()
+{
+  auto windows = GetWindows(WindowFilter::ON_CURRENT_DESKTOP | WindowFilter::ON_ALL_MONITORS);
+  auto sorted_windows = WindowManager::Default().GetWindowsInStackingOrder();
+
+  // Order the windows
+  std::sort(windows.begin(), windows.end(), [&sorted_windows] (ApplicationWindowPtr const& win1, ApplicationWindowPtr const& win2) {
+    for (auto const& window : sorted_windows)
+    {
+      if (window == win1->window_id())
+        return false;
+      else if (window == win2->window_id())
+        return true;
+    }
+
+    return true;
+  });
+
+  return windows;
+}
+
 std::string ApplicationLauncherIcon::GetName() const
 {
   return "ApplicationLauncherIcon";
