@@ -91,6 +91,9 @@ const int START_DRAGICON_DURATION = 250;
 const int MOUSE_DEADZONE = 15;
 const float DRAG_OUT_PIXELS = 300.0f;
 
+const int SCROLL_AREA_HEIGHT = 24;
+const int SCROLL_FPS = 30;
+
 const std::string START_DRAGICON_TIMEOUT = "start-dragicon-timeout";
 const std::string SCROLL_TIMEOUT = "scroll-timeout";
 const std::string ANIMATION_IDLE = "animation-idle";
@@ -101,7 +104,7 @@ NUX_IMPLEMENT_OBJECT_TYPE(Launcher);
 
 const int Launcher::Launcher::ANIM_DURATION_SHORT = 125;
 
-Launcher::Launcher(nux::BaseWindow* parent,
+Launcher::Launcher(MockableBaseWindow* parent,
                    NUX_FILE_LINE_DECL)
   : View(NUX_FILE_LINE_PARAM)
 #ifdef USE_X11
@@ -147,6 +150,7 @@ Launcher::Launcher(nux::BaseWindow* parent,
   , icon_renderer(std::make_shared<ui::IconRenderer>())
 {
   m_Layout = new nux::HLayout(NUX_TRACKER_LOCATION);
+  icon_renderer->monitor = monitor();
 
   bg_effect_helper_.owner = this;
   bg_effect_helper_.enabled = false;
@@ -195,12 +199,8 @@ Launcher::Launcher(nux::BaseWindow* parent,
   icon_renderer->SetTargetSize(_icon_size, _icon_image_size, _space_between_icons);
 
   TextureCache& cache = TextureCache::GetDefault();
-  TextureCache::CreateTextureCallback cb = [&](std::string const& name, int width, int height) {
-    return nux::CreateTexture2DFromFile((PKGDATADIR"/" + name + ".png").c_str(), -1, true);
-  };
-
-  launcher_sheen_ = cache.FindTexture("dash_sheen", 0, 0, cb);
-  launcher_pressure_effect_ = cache.FindTexture("launcher_pressure_effect", 0, 0, cb);
+  launcher_sheen_ = cache.FindTexture("dash_sheen.png");
+  launcher_pressure_effect_ = cache.FindTexture("launcher_pressure_effect.png");
 
   options.changed.connect(sigc::mem_fun(this, &Launcher::OnOptionsChanged));
   monitor.changed.connect(sigc::mem_fun(this, &Launcher::OnMonitorChanged));
@@ -286,6 +286,20 @@ void Launcher::SetStateMouseOverLauncher(bool over_launcher)
   _hide_machine.SetQuirk(LauncherHideMachine::MOUSE_OVER_LAUNCHER, over_launcher);
   _hide_machine.SetQuirk(LauncherHideMachine::REVEAL_PRESSURE_PASS, false);
   _hover_machine.SetQuirk(LauncherHoverMachine::MOUSE_OVER_LAUNCHER, over_launcher);
+  tooltip_manager_.SetHover(over_launcher);
+}
+
+void Launcher::SetIconUnderMouse(AbstractLauncherIcon::Ptr const& icon)
+{
+  if (_icon_under_mouse == icon)
+    return;
+
+  if (_icon_under_mouse)
+    _icon_under_mouse->mouse_leave.emit(monitor);
+  if (icon)
+    icon->mouse_enter.emit(monitor);
+
+  _icon_under_mouse = icon;
 }
 
 bool Launcher::MouseBeyondDragThreshold() const
@@ -770,9 +784,6 @@ void Launcher::SetupRenderArg(AbstractLauncherIcon::Ptr const& icon, struct time
   arg.running_colored     = icon->GetQuirk(AbstractLauncherIcon::Quirk::URGENT);
   arg.draw_edge_only      = IconDrawEdgeOnly(icon);
   arg.active_colored      = false;
-  arg.x_rotation          = 0.0f;
-  arg.y_rotation          = 0.0f;
-  arg.z_rotation          = 0.0f;
   arg.skip                = false;
   arg.stick_thingy        = false;
   arg.keyboard_nav_hl     = false;
@@ -820,13 +831,7 @@ void Launcher::SetupRenderArg(AbstractLauncherIcon::Ptr const& icon, struct time
   // we dont need to show strays
   if (!icon->GetQuirk(AbstractLauncherIcon::Quirk::RUNNING))
   {
-    if (icon->GetQuirk(AbstractLauncherIcon::Quirk::URGENT))
-    {
-      arg.running_arrow = true;
-      arg.window_indicators = 1;
-    }
-    else
-      arg.window_indicators = 0;
+    arg.window_indicators = 0;
   }
   else
   {
@@ -834,6 +839,13 @@ void Launcher::SetupRenderArg(AbstractLauncherIcon::Ptr const& icon, struct time
       arg.window_indicators = std::max<int> (icon->WindowsOnViewport().size(), 1);
     else
       arg.window_indicators = std::max<int> (icon->WindowsForMonitor(monitor).size(), 1);
+
+    if (icon->GetIconType() == AbstractLauncherIcon::IconType::TRASH ||
+        icon->GetIconType() == AbstractLauncherIcon::IconType::DEVICE)
+    {
+      // TODO: also these icons should respect the actual windows they have
+      arg.window_indicators = 0;
+    }
   }
 
   arg.backlight_intensity = IconBackgroundIntensity(icon, current);
@@ -849,7 +861,7 @@ void Launcher::SetupRenderArg(AbstractLauncherIcon::Ptr const& icon, struct time
 
   if (icon->GetQuirk(AbstractLauncherIcon::Quirk::URGENT) && options()->urgent_animation() == URGENT_ANIMATION_WIGGLE)
   {
-    arg.z_rotation = IconUrgentWiggleValue(icon, current);
+    arg.rotation.z = IconUrgentWiggleValue(icon, current);
   }
 
   if (IsInKeyNavMode())
@@ -927,7 +939,7 @@ void Launcher::FillRenderArg(AbstractLauncherIcon::Ptr const& icon,
 
   // icon is crossing threshold, start folding
   center.z += folded_z_distance * folding_progress;
-  arg.x_rotation = animation_neg_rads * folding_progress;
+  arg.rotation.x = animation_neg_rads * folding_progress;
 
   float spacing_overlap = CLAMP((float)(center.y + (2.0f * half_size * size_modifier) + (_space_between_icons * size_modifier) - folding_threshold) / (float) _icon_size, 0.0f, 1.0f);
   float spacing = (_space_between_icons * (1.0f - spacing_overlap) + folded_spacing * spacing_overlap) * size_modifier;
@@ -1210,8 +1222,9 @@ void Launcher::OnOverlayShown(GVariant* data)
   unity::glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
+  int width, height;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
-                &overlay_identity, &can_maximise, &overlay_monitor);
+                &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
   std::string identity(overlay_identity.Str());
 
   LOG_DEBUG(logger) << "Overlay shown: " << identity
@@ -1251,8 +1264,9 @@ void Launcher::OnOverlayHidden(GVariant* data)
   unity::glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
+  int width, height;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
-                &overlay_identity, &can_maximise, &overlay_monitor);
+                &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
 
   std::string identity = overlay_identity.Str();
 
@@ -1316,7 +1330,8 @@ void Launcher::SetHidden(bool hide_launcher)
 
   TimeUtil::SetTimeStruct(&_times[TIME_AUTOHIDE], &_times[TIME_AUTOHIDE], ANIM_DURATION_SHORT);
 
-  _parent->EnableInputWindow(!hide_launcher, launcher::window_title, false, false);
+  if (nux::GetWindowThread()->IsEmbeddedWindow())
+    _parent->EnableInputWindow(!hide_launcher, launcher::window_title, false, false);
 
   if (!hide_launcher && GetActionState() == ACTION_DRAG_EXTERNAL)
     DndReset();
@@ -1382,6 +1397,7 @@ void Launcher::OnMonitorChanged(int new_monitor)
   unity::panel::Style &panel_style = panel::Style::Instance();
   Resize(nux::Point(monitor_geo.x, monitor_geo.y + panel_style.panel_height),
          monitor_geo.height - panel_style.panel_height);
+  icon_renderer->monitor = new_monitor;
 }
 
 void Launcher::UpdateOptions(Options::Ptr options)
@@ -1395,8 +1411,6 @@ void Launcher::UpdateOptions(Options::Ptr options)
 
 void Launcher::ConfigureBarrier()
 {
-  nux::Geometry geo = GetAbsoluteGeometry();
-
   float decay_responsiveness_mult = ((options()->edge_responsiveness() - 1) * .3f) + 1;
   float reveal_responsiveness_mult = ((options()->edge_responsiveness() - 1) * .025f) + 1;
 
@@ -1485,27 +1499,18 @@ void Launcher::SetHover(bool hovered)
 
 bool Launcher::MouseOverTopScrollArea()
 {
-  return _mouse_position.y < panel::Style::Instance().panel_height;
-}
-
-bool Launcher::MouseOverTopScrollExtrema()
-{
-  return _mouse_position.y == 0;
+  return _mouse_position.y < SCROLL_AREA_HEIGHT;
 }
 
 bool Launcher::MouseOverBottomScrollArea()
 {
-  return _mouse_position.y > GetGeometry().height - panel::Style::Instance().panel_height;
-}
-
-bool Launcher::MouseOverBottomScrollExtrema()
-{
-  return _mouse_position.y == GetGeometry().height - 1;
+  return _mouse_position.y >= GetGeometry().height - SCROLL_AREA_HEIGHT;
 }
 
 bool Launcher::OnScrollTimeout()
 {
   bool continue_animation = true;
+  int speed = 0;
 
   if (IsInKeyNavMode() || !_hovered ||
       GetActionState() == ACTION_DRAG_LAUNCHER)
@@ -1516,19 +1521,21 @@ bool Launcher::OnScrollTimeout()
   {
     if (_launcher_drag_delta >= _launcher_drag_delta_max)
       continue_animation = false;
-    else if (MouseOverTopScrollExtrema())
-      _launcher_drag_delta += 6;
     else
-      _launcher_drag_delta += 3;
+    {
+        speed = (SCROLL_AREA_HEIGHT - _mouse_position.y) / SCROLL_AREA_HEIGHT * SCROLL_FPS;
+        _launcher_drag_delta += speed;
+    }
   }
   else if (MouseOverBottomScrollArea())
   {
     if (_launcher_drag_delta <= _launcher_drag_delta_min)
       continue_animation = false;
-    else if (MouseOverBottomScrollExtrema())
-      _launcher_drag_delta -= 6;
     else
-      _launcher_drag_delta -= 3;
+    {
+        speed = ((_mouse_position.y + 1) - (GetGeometry().height - SCROLL_AREA_HEIGHT)) / SCROLL_AREA_HEIGHT * SCROLL_FPS;
+        _launcher_drag_delta -= speed;
+    } 
   }
   else
   {
@@ -1600,8 +1607,7 @@ void Launcher::OnIconRemoved(AbstractLauncherIcon::Ptr const& icon)
   if (icon->needs_redraw_connection.connected())
     icon->needs_redraw_connection.disconnect();
 
-  if (icon == _icon_under_mouse)
-    _icon_under_mouse = nullptr;
+  SetIconUnderMouse(AbstractLauncherIcon::Ptr());
   if (icon == _icon_mouse_down)
     _icon_mouse_down = nullptr;
   if (icon == _drag_icon)
@@ -1681,8 +1687,6 @@ void Launcher::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
 
 void Launcher::DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw)
 {
-  icon_renderer->monitor = monitor();
-
   nux::Geometry const& base = GetGeometry();
   nux::Geometry bkg_box;
   std::list<RenderArg> args;
@@ -1920,11 +1924,7 @@ bool Launcher::StartIconDragTimeout(int x, int y)
   // if we are still waiting…
   if (GetActionState() == ACTION_NONE)
   {
-    if (_icon_under_mouse)
-    {
-      _icon_under_mouse->mouse_leave.emit(monitor);
-      _icon_under_mouse = nullptr;
-    }
+    SetIconUnderMouse(AbstractLauncherIcon::Ptr());
     _initial_drag_animation = true;
     StartIconDragRequest(x, y);
   }
@@ -2163,11 +2163,7 @@ void Launcher::RecvMouseDrag(int x, int y, int dx, int dy, unsigned long button_
       GetActionState() == ACTION_NONE)
     return;
 
-  if (_icon_under_mouse)
-  {
-    _icon_under_mouse->mouse_leave.emit(monitor);
-    _icon_under_mouse = nullptr;
-  }
+  SetIconUnderMouse(AbstractLauncherIcon::Ptr());
 
   if (GetActionState() == ACTION_NONE)
   {
@@ -2219,6 +2215,7 @@ void Launcher::RecvMouseLeave(int x, int y, unsigned long button_flags, unsigned
 void Launcher::RecvMouseMove(int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
 {
   SetMousePosition(x, y);
+  tooltip_manager_.MouseMoved(_icon_under_mouse);
 
   if (!_hidden)
     UpdateChangeInMousePosition(dx, dy);
@@ -2233,9 +2230,16 @@ void Launcher::RecvMouseWheel(int /*x*/, int /*y*/, int wheel_delta, unsigned lo
     return;
 
   bool alt_pressed = nux::GetKeyModifierState(key_flags, nux::NUX_STATE_ALT);
-
   if (alt_pressed)
+  {
     ScrollLauncher(wheel_delta);
+  }
+  else if (_icon_under_mouse)
+  {
+    auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+    auto scroll_direction = (wheel_delta < 0) ? AbstractLauncherIcon::ScrollDirection::DOWN : AbstractLauncherIcon::ScrollDirection::UP;
+    _icon_under_mouse->PerformScroll(scroll_direction, timestamp);
+  }
 }
 
 void Launcher::ScrollLauncher(int wheel_delta)
@@ -2297,7 +2301,9 @@ ui::EdgeBarrierSubscriber::Result Launcher::HandleBarrierEvent(ui::PointerBarrie
   if (!apply_to_reveal)
     return ui::EdgeBarrierSubscriber::Result::IGNORED;
 
-  _hide_machine.AddRevealPressure(event->velocity);
+  if (!owner->IsFirstEvent())
+    _hide_machine.AddRevealPressure(event->velocity);
+
   return ui::EdgeBarrierSubscriber::Result::HANDLED;
 }
 
@@ -2321,7 +2327,7 @@ void Launcher::ExitKeyNavMode()
   _hover_machine.SetQuirk(LauncherHoverMachine::KEY_NAV_ACTIVE, false);
 }
 
-void Launcher::RecvQuicklistOpened(QuicklistView* quicklist)
+void Launcher::RecvQuicklistOpened(nux::ObjectPtr<QuicklistView> const& quicklist)
 {
   UScreen* uscreen = UScreen::GetDefault();
   if (uscreen->GetMonitorGeometry(monitor).IsInside(nux::Point(quicklist->GetGeometry().x, quicklist->GetGeometry().y)))
@@ -2333,7 +2339,7 @@ void Launcher::RecvQuicklistOpened(QuicklistView* quicklist)
   }
 }
 
-void Launcher::RecvQuicklistClosed(QuicklistView* quicklist)
+void Launcher::RecvQuicklistClosed(nux::ObjectPtr<QuicklistView> const& quicklist)
 {
   nux::Point pt = nux::GetWindowCompositor().GetMousePosition();
   if (!GetAbsoluteGeometry().IsInside(pt))
@@ -2365,18 +2371,7 @@ void Launcher::EventLogic()
     launcher_icon = MouseIconIntersection(_mouse_position.x, _mouse_position.y);
   }
 
-
-  if (_icon_under_mouse && (_icon_under_mouse != launcher_icon))
-  {
-    _icon_under_mouse->mouse_leave.emit(monitor);
-    _icon_under_mouse = nullptr;
-  }
-
-  if (launcher_icon && (_icon_under_mouse != launcher_icon))
-  {
-    launcher_icon->mouse_enter.emit(monitor);
-    _icon_under_mouse = launcher_icon;
-  }
+  SetIconUnderMouse(launcher_icon);
 }
 
 void Launcher::MouseDownLogic(int x, int y, unsigned long button_flags, unsigned long key_flags)
@@ -2391,6 +2386,7 @@ void Launcher::MouseDownLogic(int x, int y, unsigned long button_flags, unsigned
     sources_.AddTimeout(START_DRAGICON_DURATION, cb_func, START_DRAGICON_TIMEOUT);
 
     launcher_icon->mouse_down.emit(nux::GetEventButton(button_flags), monitor, key_flags);
+    tooltip_manager_.IconClicked();
   }
 }
 
@@ -2459,7 +2455,7 @@ void Launcher::RenderIconToTexture(nux::GraphicsEngine& GfxContext, AbstractLaun
   SetupRenderArg(icon, current, arg);
   arg.render_center = nux::Point3(roundf(texture->GetWidth() / 2.0f), roundf(texture->GetHeight() / 2.0f), 0.0f);
   arg.logical_center = arg.render_center;
-  arg.x_rotation = 0.0f;
+  arg.rotation.x = 0.0f;
   arg.running_arrow = false;
   arg.active_arrow = false;
   arg.skip = false;
@@ -2618,7 +2614,7 @@ void Launcher::ProcessDndMove(int x, int y, std::list<char*> mimes)
     SetActionState(ACTION_DRAG_EXTERNAL);
     SetStateMouseOverLauncher(true);
 
-    if (!_steal_drag)
+    if (!_steal_drag && !_dnd_data.Uris().empty())
     {
       for (auto const& it : *_model)
       {
@@ -2732,7 +2728,7 @@ void Launcher::ProcessDndDrop(int x, int y)
   else if (_dnd_hovered_icon && _drag_action != nux::DNDACTION_NONE)
   {
      if (IsOverlayOpen())
-       ubus_.SendMessage(UBUS_PLACE_VIEW_CLOSE_REQUEST);
+       ubus_.SendMessage(UBUS_OVERLAY_CLOSE_REQUEST);
 
     _dnd_hovered_icon->AcceptDrop(_dnd_data);
   }
