@@ -24,7 +24,6 @@
 
 #include <Nux/Nux.h>
 #include <Nux/HLayout.h>
-#include <Nux/BaseWindow.h>
 #include <NuxCore/Logger.h>
 #include <UnityCore/DesktopUtilities.h>
 
@@ -68,6 +67,11 @@ const std::string DBUS_INTROSPECTION =
   "      <arg type='s' name='aptdaemon_task' direction='in'/>"
   "    </method>"
   ""
+  "    <method name='UpdateLauncherIconFavoriteState'>"
+  "      <arg type='s' name='icon_uri' direction='in'/>"
+  "      <arg type='b' name='is_sticky' direction='in'/>"
+  "    </method>"
+  ""
   "  </interface>"
   "</node>";
 }
@@ -89,11 +93,16 @@ namespace
   const std::string RUNNING_APPS_URI = FavoriteStore::URI_PREFIX_UNITY + "running-apps";
   const std::string DEVICES_URI = FavoriteStore::URI_PREFIX_UNITY + "devices";
 }
+
+std::string CreateAppUriNameFromDesktopPath(const std::string &desktop_path)
+{
+  if (desktop_path.empty())
+    return "";
+
+  return FavoriteStore::URI_PREFIX_APP + DesktopUtilities::GetDesktopID(desktop_path);
 }
 
-GDBusInterfaceVTable Controller::Impl::interface_vtable =
-  { Controller::Impl::OnDBusMethodCall, NULL, NULL};
-
+}
 
 Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager)
   : parent_(parent)
@@ -110,11 +119,7 @@ Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager)
   , keynav_restore_window_(true)
   , launcher_key_press_time_(0)
   , last_dnd_monitor_(-1)
-  , super_tap_duration_(0)
-  , dbus_owner_(g_bus_own_name(G_BUS_TYPE_SESSION, DBUS_NAME.c_str(), G_BUS_NAME_OWNER_FLAGS_NONE,
-                               OnBusAcquired, nullptr, nullptr, this, nullptr))
-  , gdbus_connection_(nullptr)
-  , reg_id_(0)
+  , dbus_server_(DBUS_NAME)
 {
 #ifdef USE_X11
   edge_barriers_.options = parent_->options();
@@ -179,6 +184,10 @@ Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager)
   xdnd_manager_->dnd_started.connect(sigc::mem_fun(this, &Impl::OnDndStarted));
   xdnd_manager_->dnd_finished.connect(sigc::mem_fun(this, &Impl::OnDndFinished));
   xdnd_manager_->monitor_changed.connect(sigc::mem_fun(this, &Impl::OnDndMonitorChanged));
+
+  dbus_server_.AddObjects(DBUS_INTROSPECTION, DBUS_PATH);
+  for (auto const& obj : dbus_server_.GetObjects())
+    obj->SetMethodsCallsHandler(sigc::mem_fun(this, &Impl::OnDBusMethodCall));
 }
 
 Controller::Impl::~Impl()
@@ -191,11 +200,6 @@ Controller::Impl::~Impl()
     if (launcher_ptr)
       launcher_ptr->GetParent()->UnReference();
   }
-
-  if (gdbus_connection_ && reg_id_)
-    g_dbus_connection_unregister_object(gdbus_connection_, reg_id_);
-
-  g_bus_unown_name(dbus_owner_);
 }
 
 void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> const& monitors)
@@ -216,7 +220,7 @@ void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> c
       launchers[i] = nux::ObjectPtr<Launcher>(CreateLauncher());
     }
 
-    int monitor = (num_launchers == 1) ? primary : i;
+    int monitor = (num_launchers == 1 && num_monitors > 1) ? primary : i;
 
     if (launchers[i]->monitor() != monitor)
     {
@@ -314,7 +318,7 @@ void Controller::Impl::OnDndMonitorChanged(int monitor)
 
 Launcher* Controller::Impl::CreateLauncher()
 {
-  nux::BaseWindow* launcher_window = new nux::BaseWindow(TEXT("LauncherWindow"));
+  auto* launcher_window = new MockableBaseWindow(TEXT("LauncherWindow"));
 
   Launcher* launcher = new Launcher(launcher_window);
   launcher->options = parent_->options();
@@ -329,7 +333,10 @@ Launcher* Controller::Impl::CreateLauncher()
   launcher_window->SetLayout(layout);
   launcher_window->SetBackgroundColor(nux::color::Transparent);
   launcher_window->ShowWindow(true);
-  launcher_window->EnableInputWindow(true, launcher::window_title, false, false);
+
+  if (nux::GetWindowThread()->IsEmbeddedWindow())
+    launcher_window->EnableInputWindow(true, launcher::window_title, false, false);
+
   launcher_window->InputWindowEnableStruts(parent_->options()->hide_mode == LAUNCHER_HIDE_NEVER);
   launcher_window->SetEnterFocusInputArea(launcher);
 
@@ -348,7 +355,7 @@ void Controller::Impl::OnLauncherAddRequest(std::string const& icon_uri, Abstrac
   if (icon_uri.find(FavoriteStore::URI_PREFIX_FILE) == 0)
   {
     auto const& desktop_path = icon_uri.substr(FavoriteStore::URI_PREFIX_FILE.length());
-    app_uri = FavoriteStore::URI_PREFIX_APP + DesktopUtilities::GetDesktopID(desktop_path);
+    app_uri = local::CreateAppUriNameFromDesktopPath(desktop_path);
   }
 
   auto const& icon = GetIconByUri(app_uri.empty() ? icon_uri : app_uri);
@@ -432,6 +439,62 @@ void Controller::Impl::SaveIconsOrder()
   FavoriteStore::Instance().SetFavorites(icons);
 }
 
+void
+Controller::Impl::OnLauncherUpdateIconStickyState(std::string const& icon_uri, bool sticky)
+{
+  if (icon_uri.empty())
+    return;
+
+  std::string target_uri = icon_uri;
+  if (icon_uri.find(FavoriteStore::URI_PREFIX_FILE) == 0)
+  {
+    auto const& desktop_path =
+      icon_uri.substr(FavoriteStore::URI_PREFIX_FILE.length());
+
+    // app uri instead
+    target_uri = local::CreateAppUriNameFromDesktopPath(desktop_path);
+  }
+  auto const& existing_icon_entry =
+    GetIconByUri(target_uri);
+
+  if (existing_icon_entry)
+    {
+      // use the backgroung mechanism of model updates & propagation
+      bool should_update = (existing_icon_entry->IsSticky() != sticky);
+      if (should_update)
+        {
+          if (sticky)
+          {
+            existing_icon_entry->Stick(true);
+          }
+          else
+          {
+            existing_icon_entry->UnStick();
+          }
+
+          SortAndUpdate();
+        }
+    }
+    else
+    {
+      FavoriteStore& favorite_store = FavoriteStore::Instance();
+
+      bool should_update = (favorite_store.IsFavorite(target_uri) != sticky);
+      if (should_update)
+        {
+          if (sticky)
+            {
+              favorite_store.AddFavorite(target_uri, -1);
+              RegisterIcon(CreateFavoriteIcon(target_uri));
+            }
+          else
+            {
+              favorite_store.RemoveFavorite(target_uri);
+            }
+        }
+    }
+}
+
 void
 Controller::Impl::OnLauncherAddRequestSpecial(std::string const& path,
                                               std::string const& aptdaemon_trans_id,
@@ -525,7 +588,7 @@ void Controller::Impl::OnLauncherEntryRemoteRemoved(LauncherEntryRemote::Ptr con
 
 void Controller::Impl::OnFavoriteStoreFavoriteAdded(std::string const& entry, std::string const& pos, bool before)
 {
-  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI || entry == expo_icon_->RemoteUri())
+  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI)
   {
     // Since the running apps and the devices are always shown, when added to
     // the model, we only have to re-order them
@@ -570,11 +633,12 @@ void Controller::Impl::OnFavoriteStoreFavoriteAdded(std::string const& entry, st
 
 void Controller::Impl::OnFavoriteStoreFavoriteRemoved(std::string const& entry)
 {
-  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI || entry == expo_icon_->RemoteUri())
+  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI)
   {
     // Since the running apps and the devices are always shown, when added to
     // the model, we only have to re-order them
     ResetIconPriorities();
+    SaveIconsOrder();
     return;
   }
 
@@ -595,7 +659,6 @@ void Controller::Impl::ResetIconPriorities()
   auto const& apps_icons = model_->GetSublist<ApplicationLauncherIcon>();
   auto const& volumes_icons = model_->GetSublist<VolumeLauncherIcon>();
   bool running_apps_found = false;
-  bool expo_icon_found = false;
   bool volumes_found = false;
 
   for (auto const& fav : favs)
@@ -622,10 +685,6 @@ void Controller::Impl::ResetIconPriorities()
       volumes_found = true;
       continue;
     }
-    else if (fav == expo_icon_->RemoteUri())
-    {
-      expo_icon_found = true;
-    }
 
     auto const& icon = GetIconByUri(fav);
 
@@ -642,9 +701,6 @@ void Controller::Impl::ResetIconPriorities()
     }
   }
 
-  if (!expo_icon_found)
-    expo_icon_->SetSortPriority(++sort_priority_);
-
   if (!volumes_found)
   {
     for (auto const& ico : volumes_icons)
@@ -655,9 +711,6 @@ void Controller::Impl::ResetIconPriorities()
   }
 
   model_->Sort();
-
-  if (!expo_icon_found)
-    SaveIconsOrder();
 }
 
 void Controller::Impl::UpdateNumWorkspaces(int workspaces)
@@ -825,8 +878,6 @@ AbstractLauncherIcon::Ptr Controller::Impl::CreateFavoriteIcon(std::string const
     if (!app || app->seen())
       return result;
 
-    // Sticky apps are those that are in the launcher when not running.
-    app->sticky = true;
     result = AbstractLauncherIcon::Ptr(new ApplicationLauncherIcon(app));
   }
   else if (icon_uri.find(FavoriteStore::URI_PREFIX_DEVICE) == 0)
@@ -923,12 +974,33 @@ void Controller::Impl::AddDevices()
   }
 }
 
+void Controller::Impl::MigrateFavorites()
+{
+  // This migrates favorites to new format, ensuring that upgrades won't lose anything
+  auto& favorites = FavoriteStore::Instance();
+  auto const& favs = favorites.GetFavorites();
+
+  auto fav_it = std::find_if(begin(favs), end(favs), [](std::string const& fav) {
+    return (fav.find(FavoriteStore::URI_PREFIX_UNITY) != std::string::npos);
+  });
+
+  if (fav_it == end(favs))
+  {
+    favorites.AddFavorite(local::RUNNING_APPS_URI, -1);
+    favorites.AddFavorite(expo_icon_->RemoteUri(), -1);
+    favorites.AddFavorite(local::DEVICES_URI, -1);
+  }
+}
+
 void Controller::Impl::SetupIcons()
 {
+  MigrateFavorites();
+
   auto& favorite_store = FavoriteStore::Instance();
   FavoriteList const& favs = favorite_store.GetFavorites();
   bool running_apps_added = false;
   bool devices_added = false;
+
   for (auto const& fav_uri : favs)
   {
     if (fav_uri == local::RUNNING_APPS_URI)
@@ -956,23 +1028,15 @@ void Controller::Impl::SetupIcons()
     AddRunningApps();
   }
 
-  if (model_->IconIndex(expo_icon_) < 0)
-  {
-    LOG_INFO(logger) << "Adding expo icon";
-    RegisterIcon(CreateFavoriteIcon(expo_icon_->RemoteUri()), ++sort_priority_);
-  }
-
   if (!devices_added)
   {
     LOG_INFO(logger) << "Adding devices";
     AddDevices();
   }
 
-  if (std::find(favs.begin(), favs.end(), expo_icon_->RemoteUri()) == favs.end())
-    SaveIconsOrder();
-
   ApplicationManager::Default().application_started
     .connect(sigc::mem_fun(this, &Impl::OnApplicationStarted));
+
   device_section_.icon_added.connect(sigc::mem_fun(this, &Impl::OnDeviceIconAdded));
   favorite_store.favorite_added.connect(sigc::mem_fun(this, &Impl::OnFavoriteStoreFavoriteAdded));
   favorite_store.favorite_removed.connect(sigc::mem_fun(this, &Impl::OnFavoriteStoreFavoriteRemoved));
@@ -990,7 +1054,7 @@ void Controller::Impl::SendHomeActivationRequest()
 }
 
 Controller::Controller(XdndManager::Ptr const& xdnd_manager)
- : options(Options::Ptr(new Options()))
+ : options(std::make_shared<Options>())
  , multiple_launchers(true)
  , pimpl(new Impl(this, xdnd_manager))
 {
@@ -1104,7 +1168,7 @@ void Controller::HandleLauncherKeyPress(int when)
 
     return false;
   };
-  pimpl->sources_.AddTimeout(pimpl->super_tap_duration_, show_launcher, local::KEYPRESS_TIMEOUT);
+  pimpl->sources_.AddTimeout(options()->super_tap_duration, show_launcher, local::KEYPRESS_TIMEOUT);
 
   auto show_shortcuts = [&]()
   {
@@ -1124,7 +1188,7 @@ void Controller::HandleLauncherKeyPress(int when)
 
 bool Controller::AboutToShowDash(int was_tap, int when) const
 {
-  if ((when - pimpl->launcher_key_press_time_) < pimpl->super_tap_duration_ && was_tap)
+  if ((when - pimpl->launcher_key_press_time_) < options()->super_tap_duration && was_tap)
     return true;
   return false;
 }
@@ -1132,7 +1196,7 @@ bool Controller::AboutToShowDash(int was_tap, int when) const
 void Controller::HandleLauncherKeyRelease(bool was_tap, int when)
 {
   int tap_duration = when - pimpl->launcher_key_press_time_;
-  if (tap_duration < pimpl->super_tap_duration_ && was_tap)
+  if (tap_duration < options()->super_tap_duration && was_tap)
   {
     LOG_DEBUG(logger) << "Quick tap, sending activation request.";
     pimpl->SendHomeActivationRequest();
@@ -1181,7 +1245,7 @@ void Controller::HandleLauncherKeyRelease(bool was_tap, int when)
   }
 }
 
-bool Controller::HandleLauncherKeyEvent(Display *display, unsigned int key_sym, unsigned long key_code, unsigned long key_state, char* key_string)
+bool Controller::HandleLauncherKeyEvent(Display *display, unsigned int key_sym, unsigned long key_code, unsigned long key_state, char* key_string, Time timestamp)
 {
   LauncherModel::iterator it;
 
@@ -1197,9 +1261,9 @@ bool Controller::HandleLauncherKeyEvent(Display *display, unsigned int key_sym,
       if (TimeUtil::TimeDelta(&current, &last_action_time) > local::ignore_repeat_shortcut_duration)
       {
         if (g_ascii_isdigit((gchar)(*it)->GetShortcut()) && (key_state & ShiftMask))
-          (*it)->OpenInstance(ActionArg(ActionArg::LAUNCHER, 0));
+          (*it)->OpenInstance(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
         else
-          (*it)->Activate(ActionArg(ActionArg::LAUNCHER, 0));
+          (*it)->Activate(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
       }
 
       // disable the "tap on super" check
@@ -1319,8 +1383,10 @@ void Controller::KeyNavTerminate(bool activate)
 
   if (activate)
   {
-    pimpl->sources_.AddIdle([this] {
-      pimpl->model_->Selection()->Activate(ActionArg(ActionArg::LAUNCHER, 0));
+    auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+
+    pimpl->sources_.AddIdle([this, timestamp] {
+      pimpl->model_->Selection()->Activate(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
       return false;
     });
   }
@@ -1345,11 +1411,6 @@ bool Controller::IsOverlayOpen() const
   return false;
 }
 
-void Controller::UpdateSuperTapDuration(int const super_tap_duration)
-{
-  pimpl->super_tap_duration_ = super_tap_duration;
-}
-
 std::string
 Controller::GetName() const
 {
@@ -1420,10 +1481,12 @@ void Controller::Impl::ReceiveLauncherKeyPress(unsigned long eventType,
 
       // <SPACE> (open a new instance)
     case NUX_VK_SPACE:
-      model_->Selection()->OpenInstance(ActionArg(ActionArg::LAUNCHER, 0));
+    {
+      auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+      model_->Selection()->OpenInstance(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
       parent_->KeyNavTerminate(false);
       break;
-
+    }
       // <RETURN> (start/activate currently selected icon)
     case NUX_VK_ENTER:
     case NUX_KP_ENTER:
@@ -1450,50 +1513,29 @@ void Controller::Impl::OpenQuicklist()
   }
 }
 
-void Controller::Impl::OnBusAcquired(GDBusConnection* connection, const gchar* name, gpointer user_data)
+GVariant* Controller::Impl::OnDBusMethodCall(std::string const& method, GVariant *parameters)
 {
-  GDBusNodeInfo* introspection_data = g_dbus_node_info_new_for_xml(DBUS_INTROSPECTION.c_str(), nullptr);
-
-  if (!introspection_data)
+  if (method == "AddLauncherItemFromPosition")
   {
-    LOG_WARNING(logger) << "No introspection data loaded. Won't get dynamic launcher addition.";
-    return;
-  }
-
-  auto self = static_cast<Controller::Impl*>(user_data);
-
-  self->gdbus_connection_ = connection;
-  self->reg_id_ = g_dbus_connection_register_object(connection, DBUS_PATH.c_str(),
-                                                    introspection_data->interfaces[0],
-                                                    &interface_vtable, user_data,
-                                                    nullptr, nullptr);
-  if (!self->reg_id_)
-  {
-    LOG_WARNING(logger) << "Object registration failed. Won't get dynamic launcher addition.";
-  }
-
-  g_dbus_node_info_unref(introspection_data);
-}
-
-void Controller::Impl::OnDBusMethodCall(GDBusConnection* connection, const gchar* sender,
-                                        const gchar* object_path, const gchar* interface_name,
-                                        const gchar* method_name, GVariant* parameters,
-                                        GDBusMethodInvocation* invocation, gpointer user_data)
-{
-  if (g_strcmp0(method_name, "AddLauncherItemFromPosition") == 0)
-  {
-    auto self = static_cast<Controller::Impl*>(user_data);
     glib::String icon, icon_title, desktop_file, aptdaemon_task;
     gint icon_x, icon_y, icon_size;
 
     g_variant_get(parameters, "(ssiiiss)", &icon_title, &icon, &icon_x, &icon_y,
                                            &icon_size, &desktop_file, &aptdaemon_task);
 
-    self->OnLauncherAddRequestSpecial(desktop_file.Str(), aptdaemon_task.Str(),
-                                      icon.Str(), icon_x, icon_y, icon_size);
-
-    g_dbus_method_invocation_return_value(invocation, nullptr);
+    OnLauncherAddRequestSpecial(desktop_file.Str(), aptdaemon_task.Str(),
+                                icon.Str(), icon_x, icon_y, icon_size);
   }
+  else if (method == "UpdateLauncherIconFavoriteState")
+  {
+    gboolean is_sticky;
+    glib::String icon_uri;
+    g_variant_get(parameters, "(sb)", &icon_uri, &is_sticky);
+
+    OnLauncherUpdateIconStickyState(icon_uri.Str(), is_sticky);
+  }
+
+  return nullptr;
 }
 
 } // namespace launcher
