@@ -62,6 +62,8 @@ const std::string UNITY_THEME_NAME = "unity-icon-theme";
 const std::string CENTER_STABILIZE_TIMEOUT = "center-stabilize-timeout";
 const std::string PRESENT_TIMEOUT = "present-timeout";
 const std::string QUIRK_DELAY_TIMEOUT = "quirk-delay-timeout";
+
+const unsigned TOOLTIP_FADE_DURATION = 80;
 }
 
 NUX_IMPLEMENT_OBJECT_TYPE(LauncherIcon);
@@ -86,6 +88,7 @@ LauncherIcon::LauncherIcon(IconType type)
   , _parent_geo(max_num_monitors)
   , _saved_center(max_num_monitors)
   , _allow_quicklist_to_show(true)
+  , _tooltip_fade_animator(TOOLTIP_FADE_DURATION)
 {
   for (unsigned i = 0; i < unsigned(Quirk::LAST); ++i)
   {
@@ -106,6 +109,7 @@ LauncherIcon::LauncherIcon(IconType type)
   tooltip_text = "blank";
 
   position = Position::FLOATING;
+  removed = false;
 
   // FIXME: the abstraction is already broken, should be fixed for O
   // right now, hooking the dynamic quicklist the less ugly possible way
@@ -114,6 +118,16 @@ LauncherIcon::LauncherIcon(IconType type)
   mouse_down.connect(sigc::mem_fun(this, &LauncherIcon::RecvMouseDown));
   mouse_up.connect(sigc::mem_fun(this, &LauncherIcon::RecvMouseUp));
   mouse_click.connect(sigc::mem_fun(this, &LauncherIcon::RecvMouseClick));
+
+  _tooltip_fade_animator.updated.connect([this] (double opacity) {
+    if (_tooltip)
+    {
+      _tooltip->SetOpacity(opacity);
+
+      if (opacity == 0.0f && _tooltip_fade_animator.GetStartValue() > _tooltip_fade_animator.GetFinishValue())
+        _tooltip->ShowWindow(false);
+    }
+  });
 }
 
 LauncherIcon::~LauncherIcon()
@@ -142,6 +156,7 @@ LauncherIcon::~LauncherIcon()
 void LauncherIcon::LoadTooltip()
 {
   _tooltip = new Tooltip();
+  _tooltip->SetOpacity(0.0f);
   AddChild(_tooltip.GetPointer());
 
   _tooltip->text = tooltip_text();
@@ -157,7 +172,7 @@ void LauncherIcon::LoadQuicklist()
     _allow_quicklist_to_show = false;
   });
 
-  QuicklistManager::Default()->RegisterQuicklist(_quicklist.GetPointer());
+  QuicklistManager::Default()->RegisterQuicklist(_quicklist);
 }
 
 const bool
@@ -230,7 +245,7 @@ LauncherIcon::OpenInstance(ActionArg arg)
   if (wm.IsScaleActive())
     wm.TerminateScale();
 
-  OpenInstanceLauncherIcon();
+  OpenInstanceLauncherIcon(arg.timestamp);
 
   UpdateQuirkTime(Quirk::LAST_ACTION);
 }
@@ -521,29 +536,23 @@ LauncherIcon::ShowTooltip()
   _tooltip->ShowTooltipWithTipAt(tip_x, tip_y);
   _tooltip->ShowWindow(!tooltip_text().empty());
   tooltip_visible.emit(_tooltip);
+
+  if (_tooltip_fade_animator.CurrentState() == nux::animation::Animation::State::Running)
+    _tooltip_fade_animator.Reverse();
+  else
+    _tooltip_fade_animator.SetStartValue(0.0f).SetFinishValue(1.0f).Start();
 }
 
 void
 LauncherIcon::RecvMouseEnter(int monitor)
 {
   _last_monitor = monitor;
-  if (QuicklistManager::Default()->Current())
-  {
-    // A quicklist is active
-    return;
-  }
-
-  ShowTooltip();
 }
 
 void LauncherIcon::RecvMouseLeave(int monitor)
 {
   _last_monitor = -1;
   _allow_quicklist_to_show = true;
-
-  if (_tooltip)
-    _tooltip->ShowWindow(false);
-  tooltip_visible.emit(nux::ObjectPtr<nux::View>(nullptr));
 }
 
 bool LauncherIcon::OpenQuicklist(bool select_first_item, int monitor)
@@ -619,13 +628,13 @@ bool LauncherIcon::OpenQuicklist(bool select_first_item, int monitor)
   {
     auto conn = std::make_shared<sigc::connection>();
     *conn = win_manager.terminate_expo.connect([this, conn, tip_x, tip_y] {
-      QuicklistManager::Default()->ShowQuicklist(_quicklist.GetPointer(), tip_x, tip_y);
+      QuicklistManager::Default()->ShowQuicklist(_quicklist, tip_x, tip_y);
       conn->disconnect();
     });
   }
   else
   {
-    QuicklistManager::Default()->ShowQuicklist(_quicklist.GetPointer(), tip_x, tip_y);
+    QuicklistManager::Default()->ShowQuicklist(_quicklist, tip_x, tip_y);
   }
 
   return true;
@@ -661,7 +670,9 @@ void LauncherIcon::RecvMouseUp(int button, int monitor, unsigned long key_flags)
 
 void LauncherIcon::RecvMouseClick(int button, int monitor, unsigned long key_flags)
 {
-  ActionArg arg(ActionArg::LAUNCHER, button);
+  auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+
+  ActionArg arg(ActionArg::Source::LAUNCHER, button, timestamp);
   arg.monitor = monitor;
 
   bool shift_pressed = nux::GetKeyModifierState(key_flags, nux::NUX_STATE_SHIFT);
@@ -677,7 +688,18 @@ void LauncherIcon::RecvMouseClick(int button, int monitor, unsigned long key_fla
 void LauncherIcon::HideTooltip()
 {
   if (_tooltip)
-    _tooltip->ShowWindow(false);
+  {
+    if (_tooltip_fade_animator.CurrentState() == nux::animation::Animation::State::Running &&
+        _tooltip_fade_animator.GetFinishValue() == 1.0)
+    {
+      _tooltip_fade_animator.Reverse();
+    }
+    else
+    {
+      _tooltip_fade_animator.SetStartValue(1.0f).SetFinishValue(0.0f).Start();
+    }
+  }
+
   tooltip_visible.emit(nux::ObjectPtr<nux::View>(nullptr));
 }
 
@@ -710,7 +732,7 @@ LauncherIcon::SetCenter(nux::Point3 const& center, int monitor, nux::Geometry co
     tip_y = new_center.y;
 
     if (_quicklist && _quicklist->IsVisible())
-      QuicklistManager::Default()->ShowQuicklist(_quicklist.GetPointer(), tip_x, tip_y);
+      QuicklistManager::Default()->ShowQuicklist(_quicklist, tip_x, tip_y);
     else if (_tooltip && _tooltip->IsVisible())
       _tooltip->ShowTooltipWithTipAt(tip_x, tip_y);
   }
@@ -1122,13 +1144,14 @@ LauncherIcon::OnRemoteCountChanged(LauncherEntryRemote* remote)
   if (!remote->CountVisible())
     return;
 
-  std::string text;
-  if (remote->Count() > 9999)
-    text = "****";
+  if (remote->Count() / 10000 != 0)
+  {
+    SetEmblemText("****");
+  }
   else
-    text = std::to_string(remote->Count());
-
-  SetEmblemText(text);
+  {
+    SetEmblemText(std::to_string(remote->Count()));
+  }
 }
 
 void
@@ -1214,6 +1237,9 @@ void LauncherIcon::UnStick()
   SetQuirk(Quirk::VISIBLE, false);
 }
 
+void LauncherIcon::PerformScroll(ScrollDirection direction, Time timestamp)
+{}
+
 
 } // namespace launcher
 } // namespace unity
