@@ -25,7 +25,6 @@
 #include <Nux/WindowCompositor.h>
 
 #include <NuxGraphics/CairoGraphics.h>
-#include <NuxGraphics/ImageSurface.h>
 #include <NuxCore/Logger.h>
 #include <UnityCore/GLibWrapper.h>
 
@@ -35,6 +34,7 @@
 #include <glib.h>
 
 #include "unity-shared/PanelStyle.h"
+#include "unity-shared/TextureCache.h"
 #include "unity-shared/WindowManager.h"
 #include "unity-shared/UBusMessages.h"
 #include <UnityCore/Variant.h>
@@ -55,8 +55,9 @@ namespace unity
 
 NUX_IMPLEMENT_OBJECT_TYPE(PanelView);
 
-PanelView::PanelView(NUX_FILE_LINE_DECL)
+PanelView::PanelView(indicator::DBusIndicators::Ptr const& remote, NUX_FILE_LINE_DECL)
   : View(NUX_FILE_LINE_PARAM)
+  , remote_(remote)
   , is_dirty_(true)
   , opacity_maximized_toggle_(false)
   , needs_geo_sync_(false)
@@ -105,7 +106,9 @@ PanelView::PanelView(NUX_FILE_LINE_DECL)
   indicators_ = new PanelIndicatorsView();
   AddPanelView(indicators_, 0);
 
-  remote_ = indicator::DBusIndicators::Ptr(new indicator::DBusIndicators());
+  for (auto const& object : remote_->GetIndicators())
+    OnObjectAdded(object);
+
   remote_->on_object_added.connect(sigc::mem_fun(this, &PanelView::OnObjectAdded));
   remote_->on_object_removed.connect(sigc::mem_fun(this, &PanelView::OnObjectRemoved));
   remote_->on_entry_activate_request.connect(sigc::mem_fun(this, &PanelView::OnEntryActivateRequest));
@@ -115,8 +118,7 @@ PanelView::PanelView(NUX_FILE_LINE_DECL)
   ubus_manager_.RegisterInterest(UBUS_BACKGROUND_COLOR_CHANGED, sigc::mem_fun(this, &PanelView::OnBackgroundUpdate));
   ubus_manager_.RegisterInterest(UBUS_OVERLAY_HIDDEN, sigc::mem_fun(this, &PanelView::OnOverlayHidden));
   ubus_manager_.RegisterInterest(UBUS_OVERLAY_SHOWN, sigc::mem_fun(this, &PanelView::OnOverlayShown));
-  ubus_manager_.RegisterInterest(UBUS_DASH_SIZE_CHANGED, [&] (GVariant *data)
-  {
+  ubus_manager_.RegisterInterest(UBUS_DASH_SIZE_CHANGED, [&] (GVariant *data) {
     int width, height;
     g_variant_get(data, "(ii)", &width, &height);
     stored_dash_width_ = width;
@@ -129,9 +131,10 @@ PanelView::PanelView(NUX_FILE_LINE_DECL)
   bg_effect_helper_.owner = this;
 
   //FIXME (gord)- replace with async loading
-  panel_sheen_.Adopt(nux::CreateTexture2DFromFile(PKGDATADIR"/dash_sheen.png", -1, true));
-  bg_refine_tex_.Adopt(nux::CreateTexture2DFromFile(PKGDATADIR"/refine_gradient_panel.png", -1, true));
-  bg_refine_single_column_tex_.Adopt(nux::CreateTexture2DFromFile(PKGDATADIR"/refine_gradient_panel_single_column.png", -1, true));
+  TextureCache& cache = TextureCache::GetDefault();
+  panel_sheen_ = cache.FindTexture("dash_sheen.png");
+  bg_refine_tex_ = cache.FindTexture("refine_gradient_panel.png");
+  bg_refine_single_column_tex_ = cache.FindTexture("refine_gradient_panel_single_column.png");
 
   rop.Blend = true;
   rop.SrcBlend = GL_ONE;
@@ -176,7 +179,7 @@ void PanelView::SetLauncherWidth(int width)
 
 bool PanelView::IsMouseInsideIndicator(nux::Point const& mouse_position) const
 {
-  return indicators_->GetGeometry().IsInside(mouse_position);
+  return indicators_->GetAbsoluteGeometry().IsInside(mouse_position);
 }
 
 void PanelView::OnBackgroundUpdate(GVariant *data)
@@ -189,7 +192,8 @@ void PanelView::OnBackgroundUpdate(GVariant *data)
   bg_color_.blue = blue;
   bg_color_.alpha = alpha;
 
-  ForceUpdateBackground();
+  if (overlay_is_open_)
+    ForceUpdateBackground();
 }
 
 void PanelView::OnOverlayHidden(GVariant* data)
@@ -197,8 +201,9 @@ void PanelView::OnOverlayHidden(GVariant* data)
   unity::glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
+  int width, height;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
-                &overlay_identity, &can_maximise, &overlay_monitor);
+                &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
 
   if (monitor_ == overlay_monitor && overlay_identity.Str() == active_overlay_)
   {
@@ -219,11 +224,13 @@ void PanelView::OnOverlayShown(GVariant* data)
   unity::glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
+  int width, height;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
-                &overlay_identity, &can_maximise, &overlay_monitor);
+                &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
 
   if (monitor_ == overlay_monitor)
   {
+    stored_dash_width_ = width;
     bg_effect_helper_.enabled = true;
     active_overlay_ = overlay_identity.Str();
     overlay_is_open_ = true;
@@ -265,7 +272,7 @@ PanelView::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
 
   GfxContext.PushClippingRectangle(geo);
 
-  if ((overlay_is_open_ || (opacity_ != 1.0f && opacity_ != 0.0f)))
+  if (IsTransparent())
   {
     nux::Geometry const& geo_absolute = GetAbsoluteGeometry();
     nux::Geometry blur_geo(geo_absolute.x, geo_absolute.y, geo.width, geo.height);
@@ -279,7 +286,7 @@ PanelView::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
       bg_blur_texture_ = bg_effect_helper_.GetRegion(blur_geo);
     }
 
-    if (bg_blur_texture_.IsValid() && (overlay_is_open_ || opacity_ != 1.0f))
+    if (bg_blur_texture_.IsValid())
     {
       nux::TexCoordXForm texxform_blur_bg;
       texxform_blur_bg.flip_v_coord = true;
@@ -369,8 +376,7 @@ PanelView::DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw)
   GfxContext.GetRenderStates().SetBlend(true);
   GfxContext.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
 
-  if (bg_blur_texture_.IsValid() &&
-      (overlay_is_open_ || (opacity_ != 1.0f && opacity_ != 0.0f)))
+  if (bg_blur_texture_.IsValid() && IsTransparent())
   {
     nux::Geometry const& geo_absolute = GetAbsoluteGeometry();
     nux::TexCoordXForm texxform_blur_bg;
@@ -480,12 +486,9 @@ PanelView::DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw)
 void
 PanelView::UpdateBackground()
 {
-  nux::Geometry const& geo = GetGeometry();
-
-  if (!is_dirty_ && geo == last_geo_)
+  if (!is_dirty_)
     return;
 
-  last_geo_ = geo;
   is_dirty_ = false;
 
   nux::ROPConfig rop;
@@ -510,13 +513,13 @@ PanelView::UpdateBackground()
         opacity = 1.0f;
     }
 
-    auto tex = panel::Style::Instance().GetBackground(geo.width, geo.height, opacity);
+    auto const& tex = panel::Style::Instance().GetBackground();
     nux::TexCoordXForm texxform;
     texxform.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
-    texxform.SetWrap(nux::TEXWRAP_REPEAT, nux::TEXWRAP_REPEAT);
+    texxform.SetWrap(nux::TEXWRAP_REPEAT, nux::TEXWRAP_CLAMP);
 
     bg_layer_.reset(new nux::TextureLayer(tex->GetDeviceTexture(), texxform,
-                                          nux::color::White, true, rop));
+                                          nux::color::White * opacity, true, rop));
   }
 }
 
@@ -525,12 +528,6 @@ void PanelView::ForceUpdateBackground()
   is_dirty_ = true;
   UpdateBackground();
 
-  indicators_->QueueDraw();
-  tray_->QueueDraw();
-
-  if (!overlay_is_open_)
-    menu_view_->QueueDraw();
-
   QueueDraw();
 }
 
@@ -645,7 +642,7 @@ void PanelView::OnEntryActivated(std::string const& entry_id, nux::Rect const& g
   }
 
   if (overlay_is_open_)
-    ubus_manager_.SendMessage(UBUS_PLACE_VIEW_CLOSE_REQUEST);
+    ubus_manager_.SendMessage(UBUS_OVERLAY_CLOSE_REQUEST);
 }
 
 void PanelView::OnEntryShowMenu(std::string const& entry_id, unsigned xid,
@@ -701,12 +698,17 @@ void PanelView::SetOpacity(float opacity)
   if (opacity_ == opacity)
     return;
 
-  opacity_ = opacity;
-  bg_effect_helper_.enabled = (opacity_ < 1.0f || overlay_is_open_);
+  opacity_ = (opacity <= 0.0f ? 0.0001f : opacity); // Not to get a black menu area
+  bg_effect_helper_.enabled = IsTransparent();
 
   ForceUpdateBackground();
 }
 
+bool PanelView::IsTransparent()
+{
+  return (opacity_ < 1.0f || overlay_is_open_);
+}
+
 void PanelView::SetMenuShowTimings(int fadein, int fadeout, int discovery,
                                    int discovery_fadein, int discovery_fadeout)
 {
@@ -787,4 +789,9 @@ bool PanelView::IsActive() const
   return menu_view_->GetControlsActive();
 }
 
+int PanelView::GetStoredDashWidth() const
+{
+  return stored_dash_width_;
+}
+
 } // namespace unity
