@@ -13,6 +13,8 @@ from autopilot.emulators.X11 import Keyboard, Mouse
 from autopilot.keybindings import KeybindingsHelper
 from testtools.matchers import GreaterThan
 
+from unity.emulators.panel import WindowButtons
+
 from unity.emulators import UnityIntrospectionObject
 import logging
 import dbus
@@ -79,6 +81,14 @@ class DashController(UnityIntrospectionObject, KeybindingsHelper):
         """Returns true if the dash is currently displaying a preview"""
         return self.view.preview_displaying;
 
+    @property
+    def preview_animation(self):
+        """Returns the average progress of dash slip and animating a preview.
+        Between 0.0 and 1.0.
+        
+        """
+        return self.view.preview_animation;
+
     def get_num_rows(self):
         """Returns the number of displayed rows in the dash."""
         return self.view.num_rows
@@ -148,6 +158,12 @@ class DashController(UnityIntrospectionObject, KeybindingsHelper):
 class DashView(UnityIntrospectionObject):
     """The dash view."""
 
+    def __get_window_buttons(self):
+        """Return the overlay window buttons view."""
+        buttons = self.get_children_by_type(OverlayWindowButtons)
+        assert(len(buttons) == 1)
+        return buttons[0]
+
     def get_searchbar(self):
         """Get the search bar attached to this dash view."""
         return self.get_children_by_type(SearchBar)[0]
@@ -170,6 +186,18 @@ class DashView(UnityIntrospectionObject):
             return preview_container
         return None
 
+    @property
+    def window_buttons(self):
+        return self.__get_window_buttons().window_buttons()
+
+
+class OverlayWindowButtons(UnityIntrospectionObject):
+    """The Overlay window buttons class"""
+
+    def window_buttons(self):
+        buttons = self.get_children_by_type(WindowButtons)
+        assert(len(buttons) == 1)
+        return buttons[0]
 
 class SearchBar(UnityIntrospectionObject):
     """The search bar for the dash view."""
