@@ -33,6 +33,9 @@ class DashTestCase(UnityTestCase):
         """Method to open the currently selected preview, if opened."""
         preview_fn = lambda: self.preview_container.current_preview
         self.assertThat(preview_fn, Eventually(NotEquals(None)))
+        self.assertThat(self.unity.dash.preview_animation, Eventually(Equals(1.0)))
+        self.assertThat(self.preview_container.animating, Eventually(Equals(False)))
+
         return preview_fn()
 
 
@@ -73,6 +76,14 @@ class DashRevealTests(DashTestCase):
         """Switch to command lens without closing the dash."""
         self.unity.dash.ensure_visible()
         self.unity.dash.reveal_command_lens()
+        self.assertThat(self.unity.dash.active_lens, Eventually(Equals('commands.lens')))
+
+    def test_command_lens_can_close_itself(self):
+        """We must be able to close the Command lens with Alt+F2"""
+        self.unity.dash.reveal_command_lens()
+        self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))
+
+        self.keybinding("lens_reveal/command")
         self.assertThat(self.unity.dash.visible, Eventually(Equals(False)))
 
     def test_alt_f4_close_dash(self):
@@ -610,14 +621,17 @@ class DashVisualTests(DashTestCase):
         """"There should be no empty space between launcher and dash when the launcher
         has a non-default width.
         """
-        self.set_unity_option('icon_size', 60)
-        self.unity.dash.ensure_visible()
-
         monitor = self.unity.dash.monitor
         launcher = self.unity.launcher.get_launcher_for_monitor(monitor)
 
+        self.set_unity_option('icon_size', 60)
+        self.assertThat(launcher.icon_size, Eventually(Equals(66)))
+
+        self.unity.dash.ensure_visible()
+
         self.assertThat(self.unity.dash.geometry[0], Eventually(Equals(launcher.geometry[0] + launcher.geometry[2] - 1)))
 
+
     def test_see_more_result_alignment(self):
         """The see more results label should be baseline aligned
         with the category name label.
@@ -969,6 +983,12 @@ class PreviewNavigateTests(DashTestCase):
 
         self.assertThat(self.unity.dash.preview_displaying, Eventually(Equals(False)))
 
+    def test_overlay_text(self):
+        """Fallback overlay text is internationalized, should always be valid."""
+        cover_art = self.get_current_preview().cover_art[0]
+        self.assertThat(cover_art.overlay_text,
+                        Eventually(Equals("No Image Available")))
+
 
 class PreviewClickCancelTests(DashTestCase):
     """Tests that the preview closes when left, middle, and right clicking in the preview"""
