@@ -22,6 +22,7 @@ from testtools.matchers import (
     LessThan,
     NotEquals,
     )
+from testtools.matchers import MismatchError
 from time import sleep
 
 from unity.emulators.icons import HudLauncherIcon
@@ -459,6 +460,17 @@ class HudBehaviorTests(HudTestsBase):
 
         self.assertProperty(char_win, is_active=True)
 
+    def test_mouse_does_not_steal_button_focus(self):
+        """When typing in the hud the mouse must not steal button focus."""
+
+        self.unity.hud.ensure_visible()
+
+        (x,y,w,h) = self.unity.hud.view.geometry
+        self.mouse.move(w/4, h/4)
+
+        self.keyboard.type("a")
+        self.assertThat(self.unity.hud.view.selected_button, Eventually(Equals(1)))
+
 
 class HudLauncherInteractionsTests(HudTestsBase):
 
@@ -705,18 +717,36 @@ class HudVisualTests(HudTestsBase):
 
 class HudAlternativeKeybindingTests(HudTestsBase):
 
+    def alternateCheck(self, keybinding="Alt", hide=False):
+        """Nasty workaround for LP: #1157738"""
+        # So, since we have no way of making sure that after changing the unity option
+        # the change will be already 'applied' on the system, let's try the keybinding
+        # a few times before deciding that it does not work and we have a regression
+        # Seems better than a sleep(some_value_here)
+        for i in range(1, 4):
+            try:
+                self.keyboard.press_and_release(keybinding)
+                self.assertThat(self.unity.hud.visible, Eventually(Equals(True), timeout=3))
+                break
+            except MismatchError:
+                continue
+        if hide:
+            self.unity.hud.ensure_hidden()
+
     def test_super_h(self):
         """Test hud reveal on <Super>h."""
+        self.addCleanup(self.alternateCheck, hide=True)
         self.set_unity_option("show_hud", "<Super>h")
         # Don't use reveal_hud, but be explicit in the keybindings.
-        self.keyboard.press_and_release("Super+h")
+        self.alternateCheck("Super+h")
         self.assertThat(self.unity.hud.visible, Eventually(Equals(True)))
 
     def test_ctrl_alt_h(self):
         """Test hud reveal on <Contrl><Alt>h."""
+        self.addCleanup(self.alternateCheck, hide=True)
         self.set_unity_option("show_hud", "<Control><Alt>h")
         # Don't use reveal_hud, but be explicit in the keybindings.
-        self.keyboard.press_and_release("Ctrl+Alt+h")
+        self.alternateCheck("Ctrl+Alt+h")
         self.assertThat(self.unity.hud.visible, Eventually(Equals(True)))
 
 
