@@ -295,8 +295,7 @@ class PanelWindowButtonsTests(PanelTestsBase):
         self.unity.dash.ensure_visible()
         self.addCleanup(self.unity.dash.ensure_hidden)
 
-        self.assertThat(self.panel.window_buttons_shown, Eventually(Equals(True)))
-        self.assertWinButtonsInOverlayMode(True)
+        self.assertThat(self.unity.dash.view.overlay_window_buttons_shown, Eventually(Equals(True)))
 
     def test_window_buttons_work_in_dash_after_launcher_resize(self):
         """When the launcher icons are resized, the window
@@ -319,14 +318,13 @@ class PanelWindowButtonsTests(PanelTestsBase):
         self.unity.hud.ensure_visible()
         self.addCleanup(self.unity.hud.ensure_hidden)
 
-        self.assertThat(self.panel.window_buttons_shown, Eventually(Equals(True)))
-        self.assertWinButtonsInOverlayMode(True)
+        self.assertThat(self.unity.hud.view.overlay_window_buttons_shown, Eventually(Equals(True)))
 
     def test_window_buttons_update_visual_state(self):
         """Window button must update its state in response to mouse events."""
-        self.unity.hud.ensure_visible()
-        self.addCleanup(self.unity.hud.ensure_hidden)
-        button = self.panel.window_buttons.close
+        self.open_new_application_window("Text Editor", maximized=True, move_to_monitor=True)
+        self.panel.move_mouse_over_window_buttons()
+        button = self.panel.window_buttons.unmaximize
 
         self.assertThat(button.visual_state, Eventually(Equals("normal")))
 
@@ -341,18 +339,17 @@ class PanelWindowButtonsTests(PanelTestsBase):
         """Window buttons must ignore clicks when the mouse released outside
         their area.
         """
-        self.unity.hud.ensure_visible()
-        self.addCleanup(self.unity.hud.ensure_hidden)
+        win = self.open_new_application_window("Text Editor", maximized=True, move_to_monitor=True)
+        self.panel.move_mouse_over_window_buttons()
 
-        button = self.panel.window_buttons.close
+        button = self.panel.window_buttons.unmaximize
         button.mouse_move_to()
         self.mouse.press()
         self.assertThat(button.visual_state, Eventually(Equals("pressed")))
         self.panel.move_mouse_below_the_panel()
         self.mouse.release()
 
-        self.assertThat(button.visual_state, Eventually(Equals("normal")))
-        self.assertThat(self.unity.hud.visible, Eventually(Equals(True)))
+        self.assertThat(win.is_maximized, Equals(True))
 
     def test_window_buttons_close_button_works_for_window(self):
         """Close window button must actually closes a window."""
@@ -519,72 +516,71 @@ class PanelWindowButtonsTests(PanelTestsBase):
         sleep(5)
         self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))
 
-    def test_window_buttons_maximization_buttons_works_for_dash(self):
-        """'Maximize' and 'Restore' buttons (when both enabled) must work as expected."""
-        # Mega-TODO:
-        #
-        # This test is terrible. The docstring is terrible. The test id is terrible.
-        # Someone needs to split this into several smaller tests. However, I'm
-        # not doing that in this branch. Consider this an invitation to split
-        # this test out and make it suck less.
-        #
-        # For your sanity I have annotated it with comments.
+    def test_window_buttons_maximize_or_restore_dash(self):
+        """Tests that the Maximize/Restore button works for the dash."""
+
         self.unity.dash.ensure_visible()
-        self.addCleanup(self.panel.window_buttons.close.mouse_click)
+        self.addCleanup(self.unity.dash.ensure_hidden)
 
-        unmaximize = self.panel.window_buttons.unmaximize
-        maximize = self.panel.window_buttons.maximize
-
-        # "netbook" means "dash is maximised"
-        dash_maximised = (self.unity.dash.view.form_factor == "netbook")
-
-        # this if statement will trigger only when we're on very small screens,
-        # where it doesn't make sense to have the dash anything other than
-        # maximised.
-        if dash_maximised and not unmaximize.enabled:
-            unmaximize.mouse_click()
-            # nice long sleep to make sure that any changes have time to process.
-            sleep(5)
-            self.assertThat(self.unity.dash.view.form_factor, Equals("netbook"))
+        desired_max = not self.unity.dash.view.dash_maximized
+        if desired_max:
+            self.panel.window_buttons.maximize.mouse_click()
         else:
-            # we are able to resize the dash.
-            # maximise and unmaximise (restore) buttons are shown in the same place
-            # but only one is shown at once:
-            if maximize.visible:
-                active_button = maximize
-                inactive_button = unmaximize
-            else:
-                active_button = unmaximize
-                inactive_button = maximize
+            self.panel.window_buttons.unmaximize.mouse_click()
 
-            self.assertThat(active_button.visible, Eventually(Equals(True)))
-            self.assertThat(active_button.sensitive, Eventually(Equals(True)))
-            self.assertThat(active_button.enabled, Eventually(Equals(True)))
-            self.assertThat(inactive_button.visible, Eventually(Equals(False)))
+        self.assertThat(self.unity.dash.view.dash_maximized, Eventually(Equals(desired_max)))
 
-            self.addCleanup(inactive_button.mouse_click)
-            active_button.mouse_click()
+    def test_window_buttons_active_inactive_states(self):
+        """Tests that the maximized/restore buttons are in the correct state when the
+        dash is open. Asserting states: visible, sensitive, enabled.
+        """
 
-            self.assertThat(inactive_button.visible, Eventually(Equals(True)))
-            self.assertThat(inactive_button.sensitive, Eventually(Equals(True)))
-            self.assertThat(inactive_button.enabled, Eventually(Equals(True)))
-            self.assertThat(active_button.visible, Eventually(Equals(False)))
+        self.unity.dash.ensure_visible()
+        self.addCleanup(self.unity.dash.ensure_hidden)
 
-            if dash_maximised:
-                self.assertThat(self.unity.dash.view.form_factor, Eventually(Equals("desktop")))
-            else:
-                self.assertThat(self.unity.dash.view.form_factor, Eventually(Equals("netbook")))
+        unmaximize = self.unity.dash.view.window_buttons.unmaximize
+        maximize = self.unity.dash.view.window_buttons.maximize
 
-            self.addCleanup(active_button.mouse_click)
-            inactive_button.mouse_click()
+        desired_max = not self.unity.dash.view.dash_maximized
+        if desired_max:
+            active   = maximize
+            inactive = unmaximize
+        else:
+            active   = unmaximize
+            inactive = maximize
 
-            self.assertThat(active_button.visible, Eventually(Equals(True)))
-            self.assertThat(inactive_button.visible, Eventually(Equals(False)))
+        self.assertThat(active.visible, Eventually(Equals(True)))
+        self.assertThat(active.sensitive, Eventually(Equals(True)))
+        self.assertThat(active.enabled, Eventually(Equals(True)))
+        self.assertThat(inactive.visible, Eventually(Equals(False)))
 
-            if dash_maximised:
-                self.assertThat(self.unity.dash.view.form_factor, Eventually(Equals("netbook")))
-            else:
-                self.assertThat(self.unity.dash.view.form_factor, Eventually(Equals("desktop")))
+    def test_window_buttons_state_switch_on_click(self):
+        """Tests that when clicking the maximize/restore window button it
+        switchs its state from either maximize to restore, or restore to
+        maximize.
+        """
+
+        self.unity.dash.ensure_visible()
+        self.addCleanup(self.unity.dash.ensure_hidden)
+
+        unmaximize = self.unity.dash.view.window_buttons.unmaximize
+        maximize = self.unity.dash.view.window_buttons.maximize
+
+        desired_max = not self.unity.dash.view.dash_maximized
+        if desired_max:
+            active   = maximize
+            inactive = unmaximize
+        else:
+            active   = unmaximize
+            inactive = maximize
+
+        active.mouse_click()
+
+        self.assertThat(inactive.visible, Eventually(Equals(True)))
+        self.assertThat(inactive.sensitive, Eventually(Equals(True)))
+        self.assertThat(inactive.enabled, Eventually(Equals(True)))
+        self.assertThat(active.visible, Eventually(Equals(False)))
+        self.assertThat(self.unity.dash.view.dash_maximized, Eventually(Equals(desired_max)))
 
     def test_minimize_button_disabled_for_non_minimizable_windows(self):
         """Minimize button must be disabled for windows that don't support minimization."""
@@ -630,6 +626,7 @@ class PanelWindowButtonsTests(PanelTestsBase):
             maximized=True,
             move_to_monitor=True)
 
+        self.sleep_menu_settle_period()
         self.assertThat(self.panel.window_buttons_shown, Eventually(Equals(False)))
 
         self.keybinding_hold("panel/show_menus")
@@ -959,6 +956,8 @@ class PanelIndicatorEntryTests(PanelTestsBase):
 
         # This assert is for timing purposes only:
         self.assertThat(menu_entry.active, Eventually(Equals(True)))
+        # Make sure we wait at least enough time that the menu appeared as well
+        sleep(self.panel.menus.fadein_duration / 1000.0)
         self.mouse.click()
 
         self.assertThat(menu_entry.active, Eventually(Equals(False)))
