@@ -42,6 +42,7 @@
 
 #include "CairoTexture.h"
 #include "JSONParser.h"
+#include "TextureCache.h"
 #include "UnitySettings.h"
 #include "config.h"
 
@@ -60,6 +61,9 @@ Style* style_instance = nullptr;
 
 const int STATES = 5;
 
+const double BUTTON_CORNER_RADIUS = 7.0;
+
+
 // These cairo overrides may also be reused somewhere...
 void cairo_set_source_rgba(cairo_t* cr, nux::Color const& color)
 {
@@ -141,6 +145,15 @@ public:
                    int       blurSize);
 
   void RoundedRectSegment(cairo_t*   cr,
+                          double     aspect,
+                          double     x,
+                          double     y,
+                          double     cornerRadius,
+                          double     width,
+                          double     height,
+                          Segment    segment);
+
+  void RoundedRectSegmentBorder(cairo_t*   cr,
                           double     aspect,
                           double     x,
                           double     y,
@@ -957,6 +970,108 @@ void Style::Impl::ButtonOutlinePath (cairo_t* cr, bool align)
 }
 
 void Style::Impl::RoundedRectSegment(cairo_t*   cr,
+                                         double     aspect,
+                                         double     x,
+                                         double     y,
+                                         double     cornerRadius,
+                                         double     width,
+                                         double     height,
+                                         Segment    segment)
+{
+  double radius  = cornerRadius / aspect;
+
+  bool odd = cairo_get_line_width (cr) == 2.0 ? false : true;
+
+  switch (segment)
+  {
+  case Segment::LEFT:
+    // top-left, right of the corner
+    cairo_move_to(cr, _align(x + radius, odd), _align(y, odd));
+
+    // top-right
+    cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
+
+    // bottom-right
+    cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+
+    // bottom-left, right of the corner
+    cairo_line_to(cr, _align(x + radius, odd), _align(y + height, odd));
+
+    // bottom-left, above the corner
+    cairo_arc(cr,
+              _align(x + radius, odd),
+              _align(y + height - radius, odd),
+              radius,
+              90.0f * G_PI / 180.0f,
+              180.0f * G_PI / 180.0f);
+
+    // left, right of the corner
+    cairo_line_to(cr, _align(x, odd), _align(y + radius, odd));
+
+    // top-left, right of the corner
+    cairo_arc(cr,
+              _align(x + radius, odd),
+              _align(y + radius, odd),
+              radius,
+              180.0f * G_PI / 180.0f,
+              270.0f * G_PI / 180.0f);
+
+    break;
+
+  case Segment::MIDDLE:
+    // top-left
+    cairo_move_to(cr, _align(x, odd), _align(y, odd));
+
+    // top-right
+    cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
+
+    // bottom-right
+    cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+
+    // bottom-left
+    cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
+
+    // back to top-left
+    cairo_close_path(cr);
+    break;
+
+  case Segment::RIGHT:
+    // top-left, right of the corner
+    cairo_move_to(cr, _align(x, odd), _align(y, odd));
+
+    // top-right, left of the corner
+    cairo_line_to(cr, _align(x + width - radius, odd), _align(y, odd));
+
+    // top-right, below the corner
+    cairo_arc(cr,
+              _align(x + width - radius, odd),
+              _align(y + radius, odd),
+              radius,
+              -90.0f * G_PI / 180.0f,
+              0.0f * G_PI / 180.0f);
+
+    // bottom-right, above the corner
+    cairo_line_to(cr, _align(x + width, odd), _align(y + height - radius, odd));
+
+    // bottom-right, left of the corner
+    cairo_arc(cr,
+              _align(x + width - radius, odd),
+              _align(y + height - radius, odd),
+              radius,
+              0.0f * G_PI / 180.0f,
+              90.0f * G_PI / 180.0f);
+
+    // bottom-left
+    cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
+
+    // back to top-left
+    cairo_close_path(cr);
+    break;
+  }
+}
+
+
+void Style::Impl::RoundedRectSegmentBorder(cairo_t*   cr,
                                          double     aspect,
                                          double     x,
                                          double     y,
@@ -982,24 +1097,29 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
     // top-right
     cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
 
-    if (arrow == Arrow::RIGHT && state == nux::VISUAL_STATE_PRESSED)
-		{
+    if (arrow == Arrow::RIGHT || arrow == Arrow::BOTH)
+    {
       cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 - arrow_h, odd));
       cairo_line_to(cr, _align(x + width - arrow_w, odd), _align(y + height / 2.0, odd));
       cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 + arrow_h, odd));
-		}
-
-    // bottom-right
-    cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+    
+      // bottom-right
+      cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+    }
+    else
+    {
+      // bottom-right
+      cairo_move_to(cr, _align(x + width, odd), _align(y + height, odd));
+    }
 
     // bottom-left, right of the corner
     cairo_line_to(cr, _align(x + radius, odd), _align(y + height, odd));
 
     // bottom-left, above the corner
     cairo_arc(cr,
-              _align(x, odd) + _align(radius, odd),
-              _align(y + height, odd) - _align(radius, odd),
-              _align(radius, odd),
+              _align(x + radius, odd),
+              _align(y + height - radius, odd),
+              radius,
               90.0f * G_PI / 180.0f,
               180.0f * G_PI / 180.0f);
 
@@ -1008,9 +1128,9 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
 
     // top-left, right of the corner
     cairo_arc(cr,
-              _align(x, odd) + _align(radius, odd),
-              _align(y, odd) + _align(radius, odd),
-              _align(radius, odd),
+              _align(x + radius, odd),
+              _align(y + radius, odd),
+              radius,
               180.0f * G_PI / 180.0f,
               270.0f * G_PI / 180.0f);
 
@@ -1024,27 +1144,33 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
     cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
 
     if ((arrow == Arrow::RIGHT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
-		{
+    {
       cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 - arrow_h, odd));
       cairo_line_to(cr, _align(x + width - arrow_w, odd), _align(y + height / 2.0, odd));
       cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 + arrow_h, odd));
-		}
-
-    // bottom-right
-    cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+      
+      // bottom-right
+      cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+    }
+    else
+    {
+      // bottom-right
+      cairo_move_to(cr, _align(x + width, odd), _align(y + height, odd));      
+    }
 
     // bottom-left
     cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
 
     if ((arrow == Arrow::LEFT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
-		{
+    {
       cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 + arrow_h, odd));
       cairo_line_to(cr, _align(x + arrow_w, odd), _align(y + height / 2.0, odd));
       cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 - arrow_h, odd));
-		}
 
-    // back to top-left
-    cairo_close_path(cr);
+      // top-left
+      cairo_line_to(cr, _align(x, odd), _align(y, odd));
+    }
+
     break;
 
   case Segment::RIGHT:
@@ -1056,9 +1182,9 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
 
     // top-right, below the corner
     cairo_arc(cr,
-              _align(x + width, odd) - _align(radius, odd),
-              _align(y, odd) + _align(radius, odd),
-              _align(radius, odd),
+              _align(x + width - radius, odd),
+              _align(y + radius, odd),
+              radius,
               -90.0f * G_PI / 180.0f,
               0.0f * G_PI / 180.0f);
 
@@ -1067,28 +1193,30 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
 
     // bottom-right, left of the corner
     cairo_arc(cr,
-              _align(x + width, odd) - _align(radius, odd),
-              _align(y + height, odd) - _align(radius, odd),
-              _align(radius, odd),
+              _align(x + width - radius, odd),
+              _align(y + height - radius, odd),
+              radius,
               0.0f * G_PI / 180.0f,
               90.0f * G_PI / 180.0f);
 
     // bottom-left
     cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
 
-    if (arrow == Arrow::LEFT && state == nux::VISUAL_STATE_PRESSED)
-		{
+    if ((arrow == Arrow::LEFT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
+    {
       cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 + arrow_h, odd));
       cairo_line_to(cr, _align(x + arrow_w, odd), _align(y + height / 2.0, odd));
       cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 - arrow_h, odd));
-		}
 
-    // back to top-left
-    cairo_close_path(cr);
+      // top-left, 
+      cairo_line_to(cr, _align(x, odd), _align(y, odd));
+    }
+
     break;
   }
 }
 
+
 void Style::Impl::ButtonOutlinePathSegment(cairo_t* cr, Segment segment)
 {
   double   x  = 0.0;
@@ -1430,11 +1558,11 @@ void Style::Impl::Text(cairo_t*    cr,
   cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
   cairo_set_source_rgba(cr, color);
   pango_layout_context_changed(layout);
-  PangoRectangle ink = {0, 0, 0, 0};
-  PangoRectangle log = {0, 0, 0, 0};
-  pango_layout_get_extents(layout, &ink, &log);
+  int layout_w = 0;
+  int layout_h = 0;
+  pango_layout_get_pixel_size(layout, &layout_w, &layout_h);
   x = horizMargin; // let pango alignment handle the x position
-  y = ((double) h - pango_units_to_double(log.height)) / 2.0;
+  y = (h - layout_h) / 2.0f;
 
   cairo_move_to(cr, x, y);
   pango_cairo_show_layout(cr, layout);
@@ -1563,7 +1691,7 @@ bool Style::Button(cairo_t* cr, nux::ButtonVisualState state,
                 1.0,
                 (double) (garnish) + 1.0,
                 (double) (garnish) + 1.0,
-                7.0,
+                BUTTON_CORNER_RADIUS,
                 w - (double) (2 * garnish) - 2.0,
                 h - (double) (2 * garnish) - 2.0);
   else
@@ -1571,7 +1699,7 @@ bool Style::Button(cairo_t* cr, nux::ButtonVisualState state,
                 1.0,
                 (double) (garnish) + 0.5,
                 (double) (garnish) + 0.5,
-                7.0,
+                BUTTON_CORNER_RADIUS,
                 w - (double) (2 * garnish) - 1.0,
                 h - (double) (2 * garnish) - 1.0);
 
@@ -1664,7 +1792,7 @@ bool Style::SquareButton(cairo_t* cr, nux::ButtonVisualState state,
     cairo_move_to(cr, _align(x + width, odd), y);
     if (curve_bottom)
     {
-      double radius = 7.0;
+      double radius = BUTTON_CORNER_RADIUS;
       LOG_DEBUG(logger) << "curve: " << _align(x + width, odd) << " - " << _align(y + height - radius, odd);
       // line to bottom-right corner
       cairo_line_to(cr, _align(x + width, odd), _align(y + height - radius, odd));
@@ -1771,7 +1899,7 @@ bool Style::ButtonFocusOverlay(cairo_t* cr, float alpha)
               1.0,
               (double) 0.5,
               (double) 0.5,
-              7.0,
+              BUTTON_CORNER_RADIUS,
               w - 1.0,
               h - 1.0);
 
@@ -1785,6 +1913,7 @@ bool Style::ButtonFocusOverlay(cairo_t* cr, float alpha)
 bool Style::MultiRangeSegment(cairo_t*    cr,
                               nux::ButtonVisualState  state,
                               std::string const& label,
+                              int font_px_size,
                               Arrow       arrow,
                               Segment     segment)
 {
@@ -1812,42 +1941,51 @@ bool Style::MultiRangeSegment(cairo_t*    cr,
     w -= 2.0;
 	}
 
-  cairo_set_line_width(cr, pimpl->button_label_border_size_[state]);
+  cairo_set_line_width(cr, pimpl->button_label_border_size_[nux::VISUAL_STATE_NORMAL]);
 
-  if (pimpl->button_label_border_size_[state] == 2.0)
-    pimpl->RoundedRectSegment(cr,
-                              1.0,
-                              x+1.0,
-                              y+1.0,
-                              (h-1.0) / 4.0,
-                              w-1.0,
-                              h-1.0,
-                              segment,
-                              arrow,
-                              state);
-  else
-    pimpl->RoundedRectSegment(cr,
-                              1.0,
-                              x,
-                              y,
-                              h / 4.0,
-                              w,
-                              h,
-                              segment,
-                              arrow,
-                              state);
+  pimpl->RoundedRectSegment(cr,
+                            1.0,
+                            x,
+                            y,
+                            BUTTON_CORNER_RADIUS,
+                            w,
+                            h,
+                            segment);
 
   if (pimpl->button_label_fill_color_[state].alpha != 0.0)
   {
     cairo_set_source_rgba(cr, pimpl->button_label_fill_color_[state]);
     cairo_fill_preserve(cr);
   }
-  cairo_set_source_rgba(cr, pimpl->button_label_border_color_[state]);
-  cairo_stroke(cr);
+
+  cairo_set_source_rgba(cr, pimpl->button_label_border_color_[nux::VISUAL_STATE_NORMAL]);
+  cairo_stroke(cr); // do not preseve path
+
+  if (state == nux::VISUAL_STATE_PRESSED)
+  {
+    int line_width = pimpl->button_label_border_size_[state];
+    cairo_set_line_width(cr, line_width);
+
+    pimpl->RoundedRectSegmentBorder(cr,
+                              1.0,
+                              x,
+                              y + line_width/2,
+                              BUTTON_CORNER_RADIUS,
+                              w,
+                              h - line_width,
+                              segment,
+                              arrow,
+                              state);
+
+    cairo_set_source_rgba(cr, pimpl->button_label_border_color_[state]);
+    cairo_stroke(cr); // do not preseve path
+  }
+
+
   pimpl->Text(cr,
               pimpl->button_label_text_color_[state],
               label,
-              10); // 13px = 10pt
+              font_px_size);
 
   return true;
 }
@@ -1888,9 +2026,7 @@ bool Style::MultiRangeFocusOverlay(cairo_t* cr,
                             h / 4.0,
                             w,
                             h,
-                            segment,
-                            arrow,
-                            nux::ButtonVisualState::VISUAL_STATE_PRESSED);
+                            segment);
 
   cairo_set_source_rgba(cr, nux::Color(1.0f, 1.0f, 1.0f, 0.5f));
   cairo_fill_preserve(cr);
@@ -2412,19 +2548,8 @@ nux::BaseTexture* LazyLoadTexture::texture()
 
 void LazyLoadTexture::LoadTexture()
 {
-  std::string full_path = PKGDATADIR + filename_;
-  glib::Object<GdkPixbuf> pixbuf;
-  glib::Error error;
-
-  pixbuf = ::gdk_pixbuf_new_from_file_at_size(full_path.c_str(), size_, size_, &error);
-  if (error)
-  {
-    LOG_WARN(logger) << "Unable to texture " << full_path << ": " << error;
-  }
-  else
-  {
-    texture_.Adopt(nux::CreateTexture2DFromPixbuf(pixbuf, true));
-  }
+  TextureCache& cache = TextureCache::GetDefault();
+  texture_ = cache.FindTexture(filename_, size_, size_);
 }
 
 } // anon namespace
