@@ -25,7 +25,9 @@
 #include <NuxGraphics/GpuDevice.h>
 #include <NuxGraphics/GLTextureResourceManager.h>
 
+#include <UnityCore/GLibWrapper.h>
 #include <NuxGraphics/CairoGraphics.h>
+#include "unity-shared/CairoTexture.h"
 #include "GraphicsUtils.h"
 
 #include <gtk/gtk.h>
@@ -39,6 +41,8 @@ namespace unity
 {
 namespace ui
 {
+namespace
+{
 
 #ifdef USE_GLES
   #define VertexShaderHeader   "#version 100\n"
@@ -172,6 +176,10 @@ MUL result.color.rgb, temp, colorify_color;                   \n\
 MOV result.color.a, color;                                    \n\
 END");
 
+const float edge_illumination_multiplier = 2.0f;
+const float glow_multiplier = 2.3f;
+} // anonymous namespace
+
 // The local namespace is purely for namespacing the file local variables below.
 namespace local
 {
@@ -181,57 +189,86 @@ enum IconSize
 {
   SMALL = 0,
   BIG,
-  LAST,
+  SIZE,
 };
+} // anonymous namespace
+} // local namespace
 
-bool textures_created = false;
-nux::BaseTexture* progress_bar_trough = 0;
-nux::BaseTexture* progress_bar_fill = 0;
-nux::BaseTexture* pip_ltr = 0;
-nux::BaseTexture* pip_rtl = 0;
-nux::BaseTexture* arrow_ltr = 0;
-nux::BaseTexture* arrow_rtl = 0;
-nux::BaseTexture* arrow_empty_ltr = 0;
-nux::BaseTexture* arrow_empty_rtl = 0;
+struct IconRenderer::TexturesPool
+{
+  static std::shared_ptr<TexturesPool> Get()
+  {
+    static std::shared_ptr<TexturesPool> instance(new TexturesPool());
+    return instance;
+  }
 
-// nux::BaseTexture* squircle_base = 0;
-// nux::BaseTexture* squircle_base_selected = 0;
-// nux::BaseTexture* squircle_edge = 0;
-// nux::BaseTexture* squircle_glow = 0;
-// nux::BaseTexture* squircle_shadow = 0;
-// nux::BaseTexture* squircle_shine = 0;
+  nux::ObjectPtr<nux::BaseTexture> RenderLabelTexture(char label, int icon_size, nux::Color const& bg_color);
 
-std::vector<nux::BaseTexture*> icon_background;
-std::vector<nux::BaseTexture*> icon_selected_background;
-std::vector<nux::BaseTexture*> icon_edge;
-std::vector<nux::BaseTexture*> icon_glow;
-std::vector<nux::BaseTexture*> icon_shadow;
-std::vector<nux::BaseTexture*> icon_shine;
-nux::ObjectPtr<nux::IOpenGLBaseTexture> offscreen_progress_texture;
-nux::ObjectPtr<nux::IOpenGLShaderProgram> shader_program_uv_persp_correction;
+  BaseTexturePtr progress_bar_trough;
+  BaseTexturePtr progress_bar_fill;
+  BaseTexturePtr pip_ltr;
+  BaseTexturePtr large_pip_ltr;
+  // BaseTexturePtr pip_rtl;
+  // BaseTexturePtr large_pip_rtl;
+  BaseTexturePtr arrow_ltr;
+  BaseTexturePtr arrow_rtl;
+  BaseTexturePtr arrow_empty_ltr;
+  // BaseTexturePtr arrow_empty_rtl;
+
+  // BaseTexturePtr squircle_base;
+  // BaseTexturePtr squircle_base_selected;
+  // BaseTexturePtr squircle_edge;
+  // BaseTexturePtr squircle_glow;
+  // BaseTexturePtr squircle_shadow;
+  // BaseTexturePtr squircle_shine;
+
+  BaseTexturePtr icon_background[local::IconSize::SIZE];
+  BaseTexturePtr icon_selected_background[local::IconSize::SIZE];
+  BaseTexturePtr icon_edge[local::IconSize::SIZE];
+  BaseTexturePtr icon_glow[local::IconSize::SIZE];
+  BaseTexturePtr icon_shadow[local::IconSize::SIZE];
+  BaseTexturePtr icon_shine[local::IconSize::SIZE];
+
+  nux::ObjectPtr<nux::IOpenGLBaseTexture> offscreen_progress_texture;
+  nux::ObjectPtr<nux::IOpenGLShaderProgram> shader_program_uv_persp_correction;
 #ifndef USE_GLES
-nux::ObjectPtr<nux::IOpenGLAsmShaderProgram> asm_shader;
+  nux::ObjectPtr<nux::IOpenGLAsmShaderProgram> asm_shader;
 #endif
-std::map<char, nux::BaseTexture*> label_map;
 
-void generate_textures();
-void destroy_textures();
-}
-}
+  int VertexLocation;
+  int VPMatrixLocation;
+  int TextureCoord0Location;
+  int FragmentColor;
+  int ColorifyColor;
+  int DesatFactor;
+
+  std::map<char, BaseTexturePtr> labels;
+
+private:
+  TexturesPool();
+
+  inline void LoadTexture(BaseTexturePtr &texture_ptr, std::string const& filename)
+  {
+    texture_ptr.Adopt(nux::CreateTexture2DFromFile(filename.c_str(), -1, true));
+  }
+
+  inline void GenerateTextures(BaseTexturePtr (&texture)[local::IconSize::SIZE],
+                               std::string const& big_file, std::string const& small_file)
+  {
+    LoadTexture(texture[local::IconSize::SMALL], small_file);
+    LoadTexture(texture[local::IconSize::BIG], big_file);
+  }
+
+  void SetupShaders();
+};
 
 IconRenderer::IconRenderer()
   : icon_size(0)
   , image_size(0)
   , spacing(0)
+  , textures_(TexturesPool::Get())
 {
   pip_style = OUTSIDE_TILE;
-
-  if (!local::textures_created)
-    local::generate_textures();
-}
-
-IconRenderer::~IconRenderer()
-{
 }
 
 void IconRenderer::SetTargetSize(int tile_size, int image_size_, int spacing_)
@@ -248,7 +285,7 @@ void IconRenderer::PreprocessIcons(std::list<RenderArg>& args, nux::Geometry con
   nux::Matrix4 ProjectionMatrix;
   nux::Matrix4 ViewProjectionMatrix;
 
-  _stored_projection_matrix = nux::GetWindowThread()->GetGraphicsEngine().GetOpenGLModelViewProjectionMatrix();
+  stored_projection_matrix_ = nux::GetWindowThread()->GetGraphicsEngine().GetOpenGLModelViewProjectionMatrix();
 
   GetInverseScreenPerspectiveMatrix(ViewMatrix, ProjectionMatrix, geo.width, geo.height, 0.1f, 1000.0f, DEGTORAD(90));
 
@@ -258,9 +295,20 @@ void IconRenderer::PreprocessIcons(std::list<RenderArg>& args, nux::Geometry con
   int i;
   for (it = args.begin(), i = 0; it != args.end(); ++it, ++i)
   {
-
     IconTextureSource* launcher_icon = it->icon;
 
+    if (it->render_center == launcher_icon->LastRenderCenter(monitor) &&
+        it->logical_center == launcher_icon->LastLogicalCenter(monitor) &&
+        it->rotation == launcher_icon->LastRotation(monitor) &&
+        it->skip == launcher_icon->WasSkipping(monitor))
+    {
+      continue;
+    }
+
+    launcher_icon->RememberCenters(monitor, it->render_center, it->logical_center);
+    launcher_icon->RememberRotation(monitor, it->rotation);
+    launcher_icon->RememberSkip(monitor, it->skip);
+
     float w = icon_size;
     float h = icon_size;
     float x = it->render_center.x - w / 2.0f; // x: top left corner
@@ -276,9 +324,9 @@ void IconRenderer::PreprocessIcons(std::list<RenderArg>& args, nux::Geometry con
     }
 
     ObjectMatrix = nux::Matrix4::TRANSLATE(geo.width / 2.0f, geo.height / 2.0f, z) * // Translate the icon to the center of the viewport
-                   nux::Matrix4::ROTATEX(it->x_rotation) *              // rotate the icon
-                   nux::Matrix4::ROTATEY(it->y_rotation) *
-                   nux::Matrix4::ROTATEZ(it->z_rotation) *
+                   nux::Matrix4::ROTATEX(it->rotation.x) *              // rotate the icon
+                   nux::Matrix4::ROTATEY(it->rotation.y) *
+                   nux::Matrix4::ROTATEZ(it->rotation.z) *
                    nux::Matrix4::TRANSLATE(-x - w / 2.0f, -y - h / 2.0f, -z); // Put the center the icon to (0, 0)
 
     ViewProjectionMatrix = PremultMatrix * ObjectMatrix;
@@ -336,9 +384,9 @@ void IconRenderer::PreprocessIcons(std::list<RenderArg>& args, nux::Geometry con
       z = it->render_center.z;
 
       ObjectMatrix = nux::Matrix4::TRANSLATE(geo.width / 2.0f, geo.height / 2.0f, z) * // Translate the icon to the center of the viewport
-                     nux::Matrix4::ROTATEX(it->x_rotation) *              // rotate the icon
-                     nux::Matrix4::ROTATEY(it->y_rotation) *
-                     nux::Matrix4::ROTATEZ(it->z_rotation) *
+                     nux::Matrix4::ROTATEX(it->rotation.x) *              // rotate the icon
+                     nux::Matrix4::ROTATEY(it->rotation.y) *
+                     nux::Matrix4::ROTATEZ(it->rotation.z) *
                      nux::Matrix4::TRANSLATE(-(it->render_center.x - w / 2.0f) - w / 2.0f, -(it->render_center.y - h / 2.0f) - h / 2.0f, -z); // Put the center the icon to (0, 0)
 
       ViewProjectionMatrix = PremultMatrix * ObjectMatrix;
@@ -404,19 +452,20 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
 
   nux::Color background_tile_color = arg.icon->BackgroundColor();
   nux::Color glow_color = arg.icon->GlowColor();
-  nux::Color edge_color(0x55555555);
+  nux::Color edge_color = nux::color::White;
   nux::Color colorify = arg.colorify;
   nux::Color background_tile_colorify = arg.colorify;
+  nux::Color edge_tile_colorify = nux::color::White;
   bool colorify_background = arg.colorify_background;
   float backlight_intensity = arg.backlight_intensity;
-  float glow_intensity = arg.glow_intensity;
+  float glow_intensity = arg.glow_intensity * glow_multiplier;
   float shadow_intensity = 0.6f;
 
-  nux::BaseTexture* background = local::icon_background[size];
-  nux::BaseTexture* edge = local::icon_edge[size];
-  nux::BaseTexture* glow = local::icon_glow[size];
-  nux::BaseTexture* shine = local::icon_shine[size];
-  nux::BaseTexture* shadow = local::icon_shadow[size];
+  BaseTexturePtr background = textures_->icon_background[size];
+  BaseTexturePtr const& edge = textures_->icon_edge[size];
+  BaseTexturePtr const& glow = textures_->icon_glow[size];
+  BaseTexturePtr const& shine = textures_->icon_shine[size];
+  BaseTexturePtr const& shadow = textures_->icon_shadow[size];
 
   nux::Color shortcut_color = arg.colorify;
 
@@ -434,6 +483,15 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
     background_tile_color = nux::color::White;
   }
 
+  if (backlight_intensity > 0 && arg.draw_edge_only)
+  {
+    float edge_glow = backlight_intensity * edge_illumination_multiplier;
+    if (edge_glow > glow_intensity)
+    {
+      glow_intensity = edge_glow;
+    }
+  }
+
   if (arg.keyboard_nav_hl)
   {
     background_tile_color = nux::color::White;
@@ -445,7 +503,7 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
     glow_intensity = 1.0f;
     shadow_intensity = 0.0f;
 
-    background = local::icon_selected_background[size];
+    background = textures_->icon_selected_background[size];
   }
   else
   {
@@ -469,7 +527,7 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
   {
     // 0.9f is BACKLIGHT_STRENGTH in Launcher.cpp
     backlight_intensity = (arg.keyboard_nav_hl) ? 0.95f : 0.9f;
-    glow_intensity = (arg.keyboard_nav_hl) ? 1.0f : 0.0f ;
+    glow_intensity = (arg.keyboard_nav_hl) ? glow_intensity : 0.0f ;
   }
 
   // draw shadow
@@ -477,8 +535,6 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
   {
     nux::Color shadow_color = background_tile_colorify * 0.3f;
 
-    // FIXME it is using the same transformation of the glow,
-    // should have its own transformation.
     RenderElement(GfxContext,
                   arg,
                   shadow->GetDeviceTexture(),
@@ -502,10 +558,9 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
                   backlight_intensity * arg.alpha,
                   force_filter,
                   tile_transform);
-  }
 
-  edge_color = edge_color + ((background_tile_color - edge_color) * backlight_intensity);
-  nux::Color edge_tile_colorify = background_tile_colorify;
+    edge_color = edge_color + ((background_tile_color - edge_color) * backlight_intensity);
+  }
 
   if (colorify_background && !arg.keyboard_nav_hl)
   {
@@ -575,7 +630,7 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
 
     RenderElement(GfxContext,
                   arg,
-                  local::icon_glow[size]->GetDeviceTexture(),
+                  textures_->icon_glow[size]->GetDeviceTexture(),
                   arg.icon->GlowColor(),
                   nux::color::White,
                   fade_out * arg.alpha,
@@ -588,17 +643,17 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
   // draw progress bar
   if (arg.progress_bias > -1.0f && arg.progress_bias < 1.0f)
   {
-    if (local::offscreen_progress_texture->GetWidth() != icon_size ||
-        local::offscreen_progress_texture->GetHeight() != icon_size)
+    if (textures_->offscreen_progress_texture->GetWidth() != icon_size ||
+        textures_->offscreen_progress_texture->GetHeight() != icon_size)
     {
-      local::offscreen_progress_texture = nux::GetGraphicsDisplay()->GetGpuDevice()
+      textures_->offscreen_progress_texture = nux::GetGraphicsDisplay()->GetGpuDevice()
         ->CreateSystemCapableDeviceTexture(icon_size, icon_size, 1, nux::BITFMT_R8G8B8A8);
     }
-    RenderProgressToTexture(GfxContext, local::offscreen_progress_texture, arg.progress, arg.progress_bias);
+    RenderProgressToTexture(GfxContext, textures_->offscreen_progress_texture, arg.progress, arg.progress_bias);
 
     RenderElement(GfxContext,
                   arg,
-                  local::offscreen_progress_texture,
+                  textures_->offscreen_progress_texture,
                   nux::color::White,
                   nux::color::White,
                   arg.alpha,
@@ -631,12 +686,22 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
   {
     char shortcut = (char) arg.shortcut_label;
 
-    if (local::label_map.find(shortcut) == local::label_map.end())
-      local::label_map[shortcut] = RenderCharToTexture(shortcut, icon_size, icon_size, shortcut_color);
+    BaseTexturePtr label;
+    auto label_it = textures_->labels.find(shortcut);
+
+    if (label_it != textures_->labels.end())
+    {
+      label = label_it->second;
+    }
+    else
+    {
+      label = textures_->RenderLabelTexture(shortcut, icon_size, shortcut_color);
+      textures_->labels[shortcut] = label;
+    }
 
     RenderElement(GfxContext,
                   arg,
-                  local::label_map[shortcut]->GetDeviceTexture(),
+                  label->GetDeviceTexture(),
                   nux::Color(0xFFFFFFFF),
                   nux::color::White,
                   arg.alpha,
@@ -645,66 +710,51 @@ void IconRenderer::RenderIcon(nux::GraphicsEngine& GfxContext, RenderArg const&
   }
 }
 
-nux::BaseTexture* IconRenderer::RenderCharToTexture(char label, int width, int height, nux::Color const& bg_color)
+nux::ObjectPtr<nux::BaseTexture> IconRenderer::TexturesPool::RenderLabelTexture(char label, int icon_size, nux::Color const& bg_color)
 {
-  nux::BaseTexture*     texture  = NULL;
-  nux::CairoGraphics*   cg       = new nux::CairoGraphics(CAIRO_FORMAT_ARGB32,
-                                                          width,
-                                                          height);
-  cairo_t*              cr       = cg->GetContext();
-  PangoLayout*          layout   = NULL;
-  PangoFontDescription* desc     = NULL;
-  GtkSettings*          settings = gtk_settings_get_default();  // not ref'ed
-  gchar*                fontName = NULL;
+  nux::CairoGraphics cg(CAIRO_FORMAT_ARGB32, icon_size, icon_size);
+  cairo_t* cr = cg.GetInternalContext();
+  glib::String font_name;
 
-  double label_ratio = 0.44f;
-  double label_size = icon_size * label_ratio;
-  double label_x = (icon_size - label_size) / 2;
-  double label_y = (icon_size - label_size) / 2;
-  double label_w = label_size;
-  double label_h = label_size;
-  double label_radius = 3.0f;
+  const double label_ratio = 0.44f;
+  const double label_size = icon_size * label_ratio;
+  const double label_x = (icon_size - label_size) / 2.0f;
+  const double label_y = (icon_size - label_size) / 2.0f;
+  const double label_w = label_size;
+  const double label_h = label_size;
+  const double label_radius = 3.0f;
 
   cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
   cairo_paint(cr);
   cairo_scale(cr, 1.0f, 1.0f);
   cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-  cg->DrawRoundedRectangle(cr, 1.0f, label_x, label_y, label_radius, label_w, label_h);
+  cg.DrawRoundedRectangle(cr, 1.0f, label_x, label_y, label_radius, label_w, label_h);
   cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.75f);
   cairo_fill_preserve(cr);
   cairo_set_source_rgba(cr, bg_color.red, bg_color.green, bg_color.blue, 0.20f);
   cairo_fill(cr);
 
-  double text_ratio = 0.75;
-  double text_size = label_size * text_ratio; 
-  layout = pango_cairo_create_layout(cr);
-  g_object_get(settings, "gtk-font-name", &fontName, NULL);
-  desc = pango_font_description_from_string(fontName);
-  pango_font_description_set_absolute_size(desc, text_size * PANGO_SCALE);
-  pango_layout_set_font_description(layout, desc);
+  const double text_ratio = 0.75;
+  double text_size = label_size * text_ratio;
+  glib::Object<PangoLayout> layout(pango_cairo_create_layout(cr));
+  g_object_get(gtk_settings_get_default(), "gtk-font-name", &font_name, NULL);
+  std::shared_ptr<PangoFontDescription> desc(pango_font_description_from_string(font_name),
+                                             pango_font_description_free);
+  pango_font_description_set_absolute_size(desc.get(), text_size * PANGO_SCALE);
+  pango_layout_set_font_description(layout, desc.get());
   pango_layout_set_text(layout, &label, 1);
 
-  PangoRectangle logRect;
-  PangoRectangle inkRect;
-  pango_layout_get_extents(layout, &inkRect, &logRect);
+  nux::Size extents;
+  pango_layout_get_pixel_size(layout, &extents.width, &extents.height);
 
   // position and paint text
   cairo_set_source_rgba(cr, 1.0f, 1.0f, 1.0f, 1.0f);
-  double x = label_x - ((logRect.width / PANGO_SCALE) - label_w) / 2.0f;
-  double y = label_y - ((logRect.height / PANGO_SCALE) - label_h) / 2.0f - 1;
+  double x = label_x - std::round((extents.width - label_w) / 2.0f);
+  double y = label_y - std::round((extents.height - label_h) / 2.0f);
   cairo_move_to(cr, x, y);
   pango_cairo_show_layout(cr, layout);
 
-  nux::NBitmapData* bitmap = cg->GetBitmap();
-  texture = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture();
-  texture->Update(bitmap);
-  delete bitmap;
-  delete cg;
-  g_object_unref(layout);
-  pango_font_description_free(desc);
-  g_free(fontName);
-
-  return texture;
+  return texture_ptr_from_cairo_graphics(cg);
 }
 
 void IconRenderer::RenderElement(nux::GraphicsEngine& GfxContext,
@@ -719,9 +769,9 @@ void IconRenderer::RenderElement(nux::GraphicsEngine& GfxContext,
   if (icon.IsNull())
     return;
 
-  if (std::abs(arg.x_rotation) < 0.01f &&
-      std::abs(arg.y_rotation) < 0.01f &&
-      std::abs(arg.z_rotation) < 0.01f &&
+  if (std::abs(arg.rotation.x) < 0.01f &&
+      std::abs(arg.rotation.y) < 0.01f &&
+      std::abs(arg.rotation.z) < 0.01f &&
       !force_filter)
   {
     icon->SetFiltering(GL_NEAREST, GL_NEAREST);
@@ -786,28 +836,23 @@ void IconRenderer::RenderElement(nux::GraphicsEngine& GfxContext,
 
   if (nux::GetWindowThread()->GetGraphicsEngine().UsingGLSLCodePath())
   {
-    local::shader_program_uv_persp_correction->Begin();
+    textures_->shader_program_uv_persp_correction->Begin();
 
-    int TextureObjectLocation   = local::shader_program_uv_persp_correction->GetUniformLocationARB("TextureObject0");
-    VertexLocation          = local::shader_program_uv_persp_correction->GetAttributeLocation("iVertex");
-    TextureCoord0Location   = local::shader_program_uv_persp_correction->GetAttributeLocation("iTexCoord0");
-    FragmentColor           = local::shader_program_uv_persp_correction->GetUniformLocationARB("color0");
-    ColorifyColor           = local::shader_program_uv_persp_correction->GetUniformLocationARB("colorify_color");
-    DesatFactor             = local::shader_program_uv_persp_correction->GetUniformLocationARB("desat_factor");
+    VertexLocation = textures_->VertexLocation;
+    TextureCoord0Location = textures_->TextureCoord0Location;
+    FragmentColor = textures_->FragmentColor;
+    ColorifyColor = textures_->ColorifyColor;
+    DesatFactor = textures_->DesatFactor;
 
-    if (TextureObjectLocation != -1)
-      CHECKGL(glUniform1iARB(TextureObjectLocation, 0));
-
-    int VPMatrixLocation = local::shader_program_uv_persp_correction->GetUniformLocationARB("ViewProjectionMatrix");
-    if (VPMatrixLocation != -1)
+    if (textures_->VPMatrixLocation != -1)
     {
-      local::shader_program_uv_persp_correction->SetUniformLocMatrix4fv((GLint)VPMatrixLocation, 1, false, (GLfloat*) & (_stored_projection_matrix.m));
+      textures_->shader_program_uv_persp_correction->SetUniformLocMatrix4fv((GLint)textures_->VPMatrixLocation, 1, false, (GLfloat*) & (stored_projection_matrix_.m));
     }
   }
 #ifndef USE_GLES
   else
   {
-    local::asm_shader->Begin();
+    textures_->asm_shader->Begin();
 
     VertexLocation        = nux::VTXATTRIB_POSITION;
     TextureCoord0Location = nux::VTXATTRIB_TEXCOORD0;
@@ -869,12 +914,12 @@ void IconRenderer::RenderElement(nux::GraphicsEngine& GfxContext,
 
   if (nux::GetWindowThread()->GetGraphicsEngine().UsingGLSLCodePath())
   {
-    local::shader_program_uv_persp_correction->End();
+    textures_->shader_program_uv_persp_correction->End();
   }
   else
   {
 #ifndef USE_GLES
-    local::asm_shader->End();
+    textures_->asm_shader->End();
 #endif
   }
 }
@@ -887,14 +932,14 @@ void IconRenderer::RenderIndicators(nux::GraphicsEngine& GfxContext,
                                     nux::Geometry const& geo)
 {
   int markerCenter = (int) arg.render_center.y;
-  markerCenter -= (int)(arg.x_rotation / (2 * M_PI) * icon_size);
+  markerCenter -= (int)(arg.rotation.x / (2 * M_PI) * icon_size);
 
 
   if (running > 0)
   {
     int scale = 1;
-
     int markerX;
+
     if (pip_style == OUTSIDE_TILE)
     {
       markerX = geo.x;
@@ -902,8 +947,7 @@ void IconRenderer::RenderIndicators(nux::GraphicsEngine& GfxContext,
     else
     {
       auto bounds = arg.icon->GetTransform(ui::IconTextureSource::TRANSFORM_TILE, monitor);
-      markerX = bounds[0].x + 2;
-      scale = 2;
+      markerX = bounds[0].x + 1;
     }
 
     nux::TexCoordXForm texxform;
@@ -917,46 +961,66 @@ void IconRenderer::RenderIndicators(nux::GraphicsEngine& GfxContext,
 
     color = color * alpha;
 
-    nux::BaseTexture* texture;
+    BaseTexturePtr texture;
 
     // markers are well outside screen bounds to start
     int markers [3] = {-100, -100, -100};
 
     if (!arg.running_on_viewport)
     {
+      scale = (pip_style == OUTSIDE_TILE) ? 1 : 2;
       markers[0] = markerCenter;
-      texture = local::arrow_empty_ltr;
+      texture = textures_->arrow_empty_ltr;
     }
     else if (running == 1)
     {
+      scale = (pip_style == OUTSIDE_TILE) ? 1 : 2;
       markers[0] = markerCenter;
-      texture = local::arrow_ltr;
+      texture = textures_->arrow_ltr;
     }
     else if (running == 2)
     {
-      markers[0] = markerCenter - 2 * scale;
-      markers[1] = markerCenter + 2 * scale;
-      texture = local::pip_ltr;
+      if (pip_style == OUTSIDE_TILE)
+      {
+        texture = textures_->pip_ltr;
+        markers[0] = markerCenter - 2;
+        markers[1] = markerCenter + 2;
+      }
+      else
+      {
+        texture = textures_->large_pip_ltr;
+        markers[0] = markerCenter - 4;
+        markers[1] = markerCenter + 4;
+      }
     }
     else
     {
-      markers[0] = markerCenter - 4 * scale;
-      markers[1] = markerCenter;
-      markers[2] = markerCenter + 4 * scale;
-      texture = local::pip_ltr;
+      if (pip_style == OUTSIDE_TILE)
+      {
+        texture = textures_->pip_ltr;
+        markers[0] = markerCenter - 4;
+        markers[1] = markerCenter;
+        markers[2] = markerCenter + 4;
+      }
+      else
+      {
+        texture = textures_->large_pip_ltr;
+        markers[0] = markerCenter - 8;
+        markers[1] = markerCenter;
+        markers[2] = markerCenter + 8;
+      }
     }
 
-
     for (int i = 0; i < 3; i++)
     {
       int center = markers[i];
       if (center == -100)
         break;
-      
+
       GfxContext.QRP_1Tex(markerX,
-                          center - ((texture->GetHeight() * scale) / 2) - 1,
-                          (float) texture->GetWidth() * scale,
-                          (float) texture->GetHeight() * scale,
+                          center - std::round((texture->GetHeight() * scale) / 2.0f),
+                          texture->GetWidth() * scale,
+                          texture->GetHeight() * scale,
                           texture->GetDeviceTexture(),
                           texxform,
                           color);
@@ -968,11 +1032,11 @@ void IconRenderer::RenderIndicators(nux::GraphicsEngine& GfxContext,
     nux::TexCoordXForm texxform;
 
     nux::Color color = nux::color::LightGrey * alpha;
-    GfxContext.QRP_1Tex((geo.x + geo.width) - local::arrow_rtl->GetWidth(),
-                        markerCenter - (local::arrow_rtl->GetHeight() / 2) - 1,
-                        (float) local::arrow_rtl->GetWidth(),
-                        (float) local::arrow_rtl->GetHeight(),
-                        local::arrow_rtl->GetDeviceTexture(),
+    GfxContext.QRP_1Tex((geo.x + geo.width) - textures_->arrow_rtl->GetWidth(),
+                        markerCenter - std::round(textures_->arrow_rtl->GetHeight() / 2.0f),
+                        textures_->arrow_rtl->GetWidth(),
+                        textures_->arrow_rtl->GetHeight(),
+                        textures_->arrow_rtl->GetDeviceTexture(),
                         texxform,
                         color);
   }
@@ -987,10 +1051,10 @@ void IconRenderer::RenderProgressToTexture(nux::GraphicsEngine& GfxContext,
   int height = texture->GetHeight();
 
   int progress_width =  icon_size;
-  int progress_height = local::progress_bar_trough->GetHeight();
+  int progress_height = textures_->progress_bar_trough->GetHeight();
 
   int fill_width = image_size - (icon_size - image_size);
-  int fill_height = local::progress_bar_fill->GetHeight();
+  int fill_height = textures_->progress_bar_fill->GetHeight();
 
   int fill_offset = (progress_width - fill_width) / 2;
 
@@ -1025,10 +1089,10 @@ void IconRenderer::RenderProgressToTexture(nux::GraphicsEngine& GfxContext,
   // left door
   GfxContext.PushClippingRectangle(nux::Geometry(left_edge, 0, half_size, height));
   GfxContext.QRP_1Tex(left_edge, progress_y, progress_width, progress_height,
-                      local::progress_bar_trough->GetDeviceTexture(), texxform,
+                      textures_->progress_bar_trough->GetDeviceTexture(), texxform,
                       nux::color::White);
   GfxContext.QRP_1Tex(left_edge + fill_offset, fill_y, fill_width, fill_height,
-                      local::progress_bar_fill->GetDeviceTexture(), texxform,
+                      textures_->progress_bar_fill->GetDeviceTexture(), texxform,
                       nux::color::White);
   GfxContext.PopClippingRectangle();
 
@@ -1036,11 +1100,11 @@ void IconRenderer::RenderProgressToTexture(nux::GraphicsEngine& GfxContext,
   GfxContext.PushClippingRectangle(nux::Geometry(left_edge + half_size, 0, half_size, height));
   GfxContext.QRP_1Tex(right_edge - progress_width, progress_y,
                       progress_width, progress_height,
-                      local::progress_bar_trough->GetDeviceTexture(), texxform,
+                      textures_->progress_bar_trough->GetDeviceTexture(), texxform,
                       nux::color::White);
   GfxContext.QRP_1Tex(right_edge - progress_width + fill_offset, fill_y,
                       fill_width, fill_height,
-                      local::progress_bar_fill->GetDeviceTexture(), texxform,
+                      textures_->progress_bar_fill->GetDeviceTexture(), texxform,
                       nux::color::White);
 
   GfxContext.PopClippingRectangle();
@@ -1048,16 +1112,9 @@ void IconRenderer::RenderProgressToTexture(nux::GraphicsEngine& GfxContext,
   unity::graphics::PopOffscreenRenderTarget();
 }
 
-void IconRenderer::DestroyTextures()
-{
-  local::destroy_textures();
-}
-
 void IconRenderer::DestroyShortcutTextures()
 {
-  for (auto texture : local::label_map)
-    texture.second->UnReference();
-  local::label_map.clear();
+  TexturesPool::Get()->labels.clear();
 }
 
 void IconRenderer::GetInverseScreenPerspectiveMatrix(nux::Matrix4& ViewMatrix, nux::Matrix4& PerspectiveMatrix,
@@ -1145,18 +1202,85 @@ void IconRenderer::GetInverseScreenPerspectiveMatrix(nux::Matrix4& ViewMatrix, n
   PerspectiveMatrix.Perspective(Fovy, AspectRatio, NearClipPlane, FarClipPlane);
 }
 
-// The local namespace is purely for namespacing the file local variables below.
-namespace local
+IconRenderer::TexturesPool::TexturesPool()
+  : offscreen_progress_texture(nux::GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableDeviceTexture(2, 2, 1, nux::BITFMT_R8G8B8A8))
+  , VertexLocation(-1)
+  , VPMatrixLocation(0)
+  , TextureCoord0Location(-1)
+  , FragmentColor(0)
+  , ColorifyColor(0)
+  , DesatFactor(0)
 {
-namespace
-{
-void setup_shaders()
+  LoadTexture(progress_bar_trough, PKGDATADIR"/progress_bar_trough.png");
+  LoadTexture(progress_bar_fill, PKGDATADIR"/progress_bar_fill.png");
+  LoadTexture(pip_ltr, PKGDATADIR"/launcher_pip_ltr.png");
+  LoadTexture(large_pip_ltr, PKGDATADIR"/launcher_pip_large_ltr.png");
+  // LoadTexture(pip_rtl, PKGDATADIR"/launcher_pip_rtl.png");
+  // LoadTexture(large_pip_rtl, PKGDATADIR"/launcher_pip_large_rtl.png");
+  LoadTexture(arrow_ltr, PKGDATADIR"/launcher_arrow_ltr.png");
+  LoadTexture(arrow_rtl, PKGDATADIR"/launcher_arrow_rtl.png");
+  LoadTexture(arrow_empty_ltr, PKGDATADIR"/launcher_arrow_outline_ltr.png");
+  // LoadTexture(arrow_empty_rtl, PKGDATADIR"/launcher_arrow_outline_rtl.png");
+
+  // LoadTexture(squircle_base, PKGDATADIR"/squircle_base_54.png");
+  // LoadTexture(squircle_base_selected, PKGDATADIR"/squircle_base_selected_54.png");
+  // LoadTexture(squircle_edge, PKGDATADIR"/squircle_edge_54.png");
+  // LoadTexture(squircle_glow, PKGDATADIR"/squircle_glow_62.png");
+  // LoadTexture(squircle_shadow, PKGDATADIR"/squircle_shadow_62.png");
+  // LoadTexture(squircle_shine, PKGDATADIR"/squircle_shine_54.png");
+
+  // BaseTexturePtr icon_background[local::IconSize::SIZE];
+  // BaseTexturePtr icon_selected_background[local::IconSize::SIZE];
+  // BaseTexturePtr icon_edge[local::IconSize::SIZE];
+  // BaseTexturePtr icon_glow[local::IconSize::SIZE];
+  // BaseTexturePtr icon_shadow[local::IconSize::SIZE];
+  // BaseTexturePtr icon_shine[local::IconSize::SIZE];
+
+  GenerateTextures(icon_background,
+                   PKGDATADIR"/launcher_icon_back_150.png",
+                   PKGDATADIR"/launcher_icon_back_54.png");
+  GenerateTextures(icon_selected_background,
+                   PKGDATADIR"/launcher_icon_selected_back_150.png",
+                   PKGDATADIR"/launcher_icon_back_54.png");
+  GenerateTextures(icon_edge,
+                   PKGDATADIR"/launcher_icon_edge_150.png",
+                   PKGDATADIR"/launcher_icon_edge_54.png");
+  GenerateTextures(icon_glow,
+                   PKGDATADIR"/launcher_icon_glow_200.png",
+                   PKGDATADIR"/launcher_icon_glow_62.png");
+  GenerateTextures(icon_shadow,
+                   PKGDATADIR"/launcher_icon_shadow_200.png",
+                   PKGDATADIR"/launcher_icon_shadow_62.png");
+  GenerateTextures(icon_shine,
+                   PKGDATADIR"/launcher_icon_shine_150.png",
+                   PKGDATADIR"/launcher_icon_shine_54.png");
+
+  SetupShaders();
+}
+
+void IconRenderer::TexturesPool::SetupShaders()
 {
   if (nux::GetWindowThread()->GetGraphicsEngine().UsingGLSLCodePath())
   {
     shader_program_uv_persp_correction = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateShaderProgram();
     shader_program_uv_persp_correction->LoadIShader(gPerspectiveCorrectShader.c_str());
     shader_program_uv_persp_correction->Link();
+
+    shader_program_uv_persp_correction->Begin();
+
+    int TextureObjectLocation = shader_program_uv_persp_correction->GetUniformLocationARB("TextureObject0");
+    VertexLocation            = shader_program_uv_persp_correction->GetAttributeLocation("iVertex");
+    TextureCoord0Location     = shader_program_uv_persp_correction->GetAttributeLocation("iTexCoord0");
+    FragmentColor             = shader_program_uv_persp_correction->GetUniformLocationARB("color0");
+    ColorifyColor             = shader_program_uv_persp_correction->GetUniformLocationARB("colorify_color");
+    DesatFactor               = shader_program_uv_persp_correction->GetUniformLocationARB("desat_factor");
+
+    if (TextureObjectLocation != -1)
+      CHECKGL(glUniform1iARB(TextureObjectLocation, 0));
+
+    VPMatrixLocation = shader_program_uv_persp_correction->GetUniformLocationARB("ViewProjectionMatrix");
+
+    shader_program_uv_persp_correction->End();
   }
   else
   {
@@ -1181,107 +1305,5 @@ void setup_shaders()
   }
 }
 
-
-inline nux::BaseTexture* load_texture(const char* filename)
-{
-  return nux::CreateTexture2DFromFile(filename, -1, true);
-}
-
-void generate_textures(std::vector<nux::BaseTexture*>& icons, const char* big_file, const char* small_file)
-{
-  icons.resize(IconSize::LAST);
-  icons[IconSize::BIG] = load_texture(big_file);
-  icons[IconSize::SMALL] = load_texture(small_file);
-}
-
-void generate_textures()
-{
-  progress_bar_trough = load_texture(PKGDATADIR"/progress_bar_trough.png");
-  progress_bar_fill = load_texture(PKGDATADIR"/progress_bar_fill.png");
-
-  generate_textures(icon_background,
-                    PKGDATADIR"/launcher_icon_back_150.png",
-                    PKGDATADIR"/launcher_icon_back_54.png");
-  generate_textures(icon_selected_background,
-                    PKGDATADIR"/launcher_icon_selected_back_150.png",
-                    PKGDATADIR"/launcher_icon_back_54.png");
-  generate_textures(icon_edge,
-                    PKGDATADIR"/launcher_icon_edge_150.png",
-                    PKGDATADIR"/launcher_icon_edge_54.png");
-  generate_textures(icon_glow,
-                    PKGDATADIR"/launcher_icon_glow_200.png",
-                    PKGDATADIR"/launcher_icon_glow_62.png");
-  generate_textures(icon_shadow,
-                    PKGDATADIR"/launcher_icon_shadow_200.png",
-                    PKGDATADIR"/launcher_icon_shadow_62.png");
-  generate_textures(icon_shine,
-                    PKGDATADIR"/launcher_icon_shine_150.png",
-                    PKGDATADIR"/launcher_icon_shine_54.png");
-
-  // squircle_base = load_texture(PKGDATADIR"/squircle_base_54.png");
-  // squircle_base_selected = load_texture(PKGDATADIR"/squircle_base_selected_54.png");
-  // squircle_edge = load_texture(PKGDATADIR"/squircle_edge_54.png");
-  // squircle_glow = load_texture(PKGDATADIR"/squircle_glow_62.png");
-  // squircle_shadow = load_texture(PKGDATADIR"/squircle_shadow_62.png");
-  // squircle_shine = load_texture(PKGDATADIR"/squircle_shine_54.png");
-
-  pip_ltr = load_texture(PKGDATADIR"/launcher_pip_ltr.png");
-  arrow_ltr = load_texture(PKGDATADIR"/launcher_arrow_ltr.png");
-  arrow_empty_ltr = load_texture(PKGDATADIR"/launcher_arrow_outline_ltr.png");
-
-  pip_rtl = load_texture(PKGDATADIR"/launcher_pip_rtl.png");
-  arrow_rtl = load_texture(PKGDATADIR"/launcher_arrow_rtl.png");
-  arrow_empty_rtl = load_texture(PKGDATADIR"/launcher_arrow_outline_rtl.png");
-
-  offscreen_progress_texture = nux::GetGraphicsDisplay()->GetGpuDevice()
-    ->CreateSystemCapableDeviceTexture(2, 2, 1, nux::BITFMT_R8G8B8A8);
-
-  setup_shaders();
-  textures_created = true;
-}
-
-void destroy_textures(std::vector<nux::BaseTexture*>& icons)
-{
-  icons[SMALL]->UnReference();
-  icons[BIG]->UnReference();
-  icons.clear();
-}
-
-void destroy_textures()
-{
-  if (!textures_created)
-    return;
-
-  progress_bar_trough->UnReference();
-  progress_bar_fill->UnReference();
-  pip_ltr->UnReference();
-  pip_rtl->UnReference();
-  arrow_ltr->UnReference();
-  arrow_rtl->UnReference();
-  arrow_empty_ltr->UnReference();
-  arrow_empty_rtl->UnReference();
-
-  destroy_textures(icon_background);
-  destroy_textures(icon_selected_background);
-  destroy_textures(icon_edge);
-  destroy_textures(icon_glow);
-  destroy_textures(icon_shadow);
-  destroy_textures(icon_shine);
-
-  // squircle_base->UnReference();
-  // squircle_base_selected->UnReference();
-  // squircle_edge->UnReference();
-  // squircle_glow->UnReference();
-  // squircle_shadow->UnReference();
-  // squircle_shine->UnReference();
-
-  IconRenderer::DestroyShortcutTextures();
-
-  textures_created = false;
-}
-
-} // anon namespace
-} // namespace local
-
 } // namespace ui
 } // namespace unity
