@@ -1113,6 +1113,26 @@ nux::Point PluginAdapter::GetCurrentViewport() const
   return nux::Point(vp.x(), vp.y());
 }
 
+void PluginAdapter::SetViewportSize(int horizontal, int vertical)
+{
+  if (horizontal < 1 || vertical < 1)
+  {
+    LOG_ERROR(logger) << "Impossible to set viewport to invalid values "
+                      << horizontal << "x" << vertical;
+    return;
+  }
+
+  CompOption::Value hsize;
+  hsize.set<int>(horizontal);
+  m_Screen->setOptionForPlugin("core", "hsize", hsize);
+
+  CompOption::Value vsize(vertical);
+  vsize.set<int>(vertical);
+  m_Screen->setOptionForPlugin("core", "vsize", vsize);
+
+  LOG_INFO(logger) << "Setting viewport size to " << hsize.i() << "x" << vsize.i();
+}
+
 int PluginAdapter::GetViewportHSize() const
 {
   return m_Screen->vpSize().width();
