@@ -29,6 +29,7 @@
 #include "WindowButtons.h"
 #include "WindowButtonPriv.h"
 
+#include "unity-shared/TextureCache.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/WindowManager.h"
 
@@ -176,8 +177,8 @@ nux::ObjectPtr<nux::BaseTexture> WindowButton::GetDashWindowButton(panel::Window
 
   std::string subpath = names[static_cast<int>(type)] + states[static_cast<int>(state)] + ".png";
 
-  glib::String filename(g_build_filename(PKGDATADIR, subpath.c_str(), NULL));
-  texture.Adopt(nux::CreateTexture2DFromFile(filename, -1, true));
+  auto& cache = TextureCache::GetDefault();
+  texture = cache.FindTexture(subpath);
 
   if (!texture)
     texture = panel::Style::Instance().GetFallbackWindowButton(type, state);
@@ -360,7 +361,7 @@ void WindowButtons::OnCloseClicked(nux::Button *button)
 
   if (win_button->overlay_mode())
   {
-    ubus_manager_.SendMessage(UBUS_PLACE_VIEW_CLOSE_REQUEST);
+    ubus_manager_.SendMessage(UBUS_OVERLAY_CLOSE_REQUEST);
   }
   else
   {
@@ -429,8 +430,9 @@ void WindowButtons::OnOverlayShown(GVariant* data)
   glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
+  int width, height;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
-                &overlay_identity, &can_maximise, &overlay_monitor);
+                &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
 
   if (overlay_monitor != monitor())
   {
@@ -500,8 +502,9 @@ void WindowButtons::OnOverlayHidden(GVariant* data)
   glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
+  int width, height;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
-                &overlay_identity, &can_maximise, &overlay_monitor);
+                &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
 
   if (overlay_monitor != monitor())
   {
